<?php

class rfc2047
{
	const ENCODED_WORD_CONSTANT_LEN = 7;		// Amount of constant characters in an encoded-word
	const ENCODED_WORD_MAX_LEN      = 75;		// Maximum number of characters in an encoded-word including constants
	const HEADER_LINE_SHORT_LIMIT   = 76;		// Maximum number of characters on a single line containing an encoded-word
	const HEADER_LINE_LONG_LIMIT	= 998;		// Maximum number of characters on a single line not containing an encoded-word
	const HEADER_LINEFEED			= "\r\n ";	// Linefeed sequence for encoded-word header continuation (CRLF SPACE)
	const HEADER_LINEFEED_LEN		= 3;		// Length of linefeed sequence

	// Returns the size, in characters that would be required to encode $word as an RFC 2047 "encoded-word"
	// Currently only supports base64
	public static function getEncodedWordSize($word, $charset)
	{
		$wordLen = strlen($word);
	    
		// The reserved length (stuff we can't change) is the length of the charset plus constants.
		// This is how many characters will be added to the base64 form of an encoded-word.
		// (e.g. =?UTF-8?B?<base64 here>?=)    
		$reservedLen = strlen($charset) + self::ENCODED_WORD_CONSTANT_LEN;

		// For every 3 bytes of input, base64 produces 4 bytes of output.  If the input data is not evenly
		// divisible by 3, there is another 4 bytes tacked on to the output (padded).
		$encodedLen  = ((floor(($wordLen + 2) / 3)) << 2);
	    
		return $reservedLen + $encodedLen;
	}

	// Returns the RFC 2047 "encoded-word" form of $word
	// Currently only supports base64
    public static function getEncodedForm($word, $charset)
    {
        return "=?" . $charset . "?B?" . base64_encode($word) . "?=";
    }

	// Returns the maximum number of bytes that can be encoded as an RFC 2047 "encoded-word" using
	// $charset and still be less than or equal to $maxLength.
	// Currently only supports base64
	public static function getMaximumEncodeBytes($charset, $maxLength = self::ENCODED_WORD_MAX_LEN)
    {
        // Reserved characters are the charset and constants required
		$reservedLen = strlen($charset) + self::ENCODED_WORD_CONSTANT_LEN;
        
		// Reserved length + 4 (one to three bytes of input as base64) is the minimium size possible.
		if ($maxLength <= ($reservedLen + 4))
		{
			// Too small to encode even one byte
			return 0;
		}

        // The maximum amount of *base64* data that can be included in an encoded-word
        // is $maxLength minus reserved chars.
        $base64Max = $maxLength - $reservedLen;
        
        // The maximum number of real characters that make up the base64 data is the
        // size of the base64 data, divided by 4, times 3. The size of the base64
        // data is always a multiple of 4.  Each set of 4 base64 characters represents 3 real characters.
        return (($base64Max >> 2) * 3);
    }
}

?>
