<?php

if (!defined("INSWIFT")) {
	trigger_error("Unable to process " . htmlentities($_SERVER['PHP_SELF']), E_USER_ERROR);
}

global $staffauth;
if (!$staffauth->isAdmin()) {
	trigger_error("You do not have permission to access " . htmlentities($_SERVER['PHP_SELF']), E_USER_ERROR);
}





 /*********
  *
  *  Writes output to the backup stream.  Pretty simple in the RAM driver.
  *
  *  @author    John Haugeland
  *  @since     2.0
  *  @version   1
  *  @access    public
  *
  *  @param     boolean     $nodash      whether or not to include hyphens in the result value
  *
  *  @return    UUID as string
  *
  *  @todo      pull into a more appropriate location (after finding it)
  *
  */

  function WriteToBackup($x) {
    echo $x;  
  }





 /*********
  *
  *  Driver entrypoint to start the backup process
  *
  *  @author    John Haugeland
  *  @since     2.0
  *  @version   1
  *  @access    public
  *
  */

  function StartBackupOutput() {
    ob_start();
  }





 /*********
  *
  *  Driver entrypoint to finish the backup process; returns the backup as held.
  *
  *  @author    John Haugeland
  *  @since     2.0
  *  @version   1
  *  @access    public
  *
  *  @return    The backup (string value)
  *
  */

  function StopBackupAndReturnOutput() {
    $contents = ob_get_contents();
    ob_end_clean();
    return $contents;
  }





 /*********
  *
  *  Generate the filename for a backup, given a database's base name and whether or not the output is to be compressed
  *
  *  @author    John Haugeland
  *  @since     2.0
  *  @version   1
  *  @access    public
  *
  *  @param		string	$basename	The name of the database
  *  @param		boolean	$compressed	Whether to write the name of a compressed or uncompressed file (to be removed in favor of a seperate driver)
  * 
  *  @return    The backup filename (string)
  * 
  *  @todo		split into two drivers, one for compressed and one for not
  *
  */

  function GenBackupFilename($basename) {
    return $basename . "." . date("d") . "_" . date("M") . "_" . date("Y") . '.sql.gz';
  }





 /*********
  *
  *  Stops the backup process and dispatches the results to the client.  Implies cleanup.
  *
  *  @author    John Haugeland
  *  @since     2.0
  *  @version   1
  *  @access    public
  *
  */

  function StopAndTransmitBackup() {
    
    global $_DB;
    
    WriteHtmlHeader(GenBackupFilename($_DB['name']));
        
    $contents           = StopBackupAndReturnOutput();
    $crc                = crc32($contents);
    $length             = strlen($contents);
    $compressedcontents = gzcompress($contents);
        
    unset($contents);
        
    // GZIP Header
    echo "\x1f\x8b\x08\x00\x00\x00\x00\x00";
        
    // Erase last 4 bytes of compressed content
    echo substr($compressedcontents, 0, strlen($compressedcontents) - 4);
        
    echo pack('V', $crc);
    echo pack('V', $length);
    // All done!
    
  }



	

?>