<?php
//=======================================
//###################################
// Kayako Infotech Ltd. - SWIFT Framework
//
// Source Copyright 2001-2008 Kayako Infotech Ltd.
// Unauthorized reproduction is not allowed
// License Number: $%LICENSE%$
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//                   www.kayako.com
//###################################
//=======================================

if (!defined("INSWIFT")) {
	trigger_error("Unable to process " . htmlentities($_SERVER['PHP_SELF']), E_USER_ERROR);
}

// Used by getCalendarDateFormats
// RML
define ("DATE_FORMAT_HTML_US_HOUR", "M d Y g:i A");
define ("DATE_FORMAT_HTML_EU_HOUR", "d M Y H:i");
define ("DATE_FORMAT_CAL_US_HOUR", "%b %d %Y %l:%M %p");
define ("DATE_FORMAT_CAL_EU_HOUR", "%d %b %Y %H:%M");
define ("DATE_FORMAT_HTML_US", "M d Y");
define ("DATE_FORMAT_HTML_EU", "d M Y");
define ("DATE_FORMAT_CAL_US", "%b %d %Y");
define ("DATE_FORMAT_CAL_EU", "%d %b %Y");
define ("DATE_FORMAT_HOUR_US", "12");
define ("DATE_FORMAT_HOUR_EU", "24");
define ("TIME_FORMAT_US", "g:i A");
define ("TIME_FORMAT_EU", "H:i");

// Used globally
// RML
define ("DATE_ONE_DAY", 86400);
define ("DATE_ONE_WEEK", 604800);

/*
* Returns the date format(s) to use when constructing a calendar control.
* 
* Return Value:
*   Array with the following keys:
*       "html": This is the format to use in the actual HTML entity's value.
*       "cal":  This is the format to pass to the calendar control.
*       "hour": This is the clock format to pass to the calendar control
*       "usformat": True if the setting is enabled to use US date/time formats
* RML
*/
function getCalendarDateFormats($useHours = true)
{
    global $_SWIFT;
    
    $retVal = array();
    $retVal["usformat"] = ($_SWIFT["settings"]["dt_caltype"] == "us");
    
    if (true == $useHours)
    {
        $retVal["html"] = ($retVal["usformat"] ? DATE_FORMAT_HTML_US_HOUR : DATE_FORMAT_HTML_EU_HOUR);
        $retVal["cal"]  = ($retVal["usformat"] ? DATE_FORMAT_CAL_US_HOUR : DATE_FORMAT_CAL_EU_HOUR);
    }
    else
    {
        $retVal["html"] = ($retVal["usformat"] ? DATE_FORMAT_HTML_US : DATE_FORMAT_HTML_EU);
        $retVal["cal"]  = ($retVal["usformat"] ? DATE_FORMAT_CAL_US : DATE_FORMAT_CAL_EU);    
    }
    
    $retVal["hour"] = ($retVal["usformat"] ? DATE_FORMAT_HOUR_US : DATE_FORMAT_HOUR_EU);
    
    return $retVal;
}

/*
* Takes a UNIX epoch value and ensures it is the lowest value for that specific day available,
* meaning 12 AM and zero seconds.
* RML
*/
function floorDate($date)
{
    return mktime(0, 0, 0, date("m", $date), date("d", $date), date("Y", $date));
}

/*
* Takes a UNIX epoch value and ensures it is the highest value for that specific day available,
* meaning 11:59 PM and 59 seconds.
* RML
*/
function ceilDate($date)
{
    return mktime(23, 59, 59, date("m", $date), date("d", $date), date("Y", $date));
}

/*
* Takes a UNIX epoch value and sets it to the lowest value available for the same month.
* e.g. May 08 2008 2:05 PM -> May 01 2008 12:00 AM
* RML
*/
function firstOfTheMonth($date)
{
    return mktime(0, 0, 0, date("m", $date), 1, date("Y", $date));
}

/*
* Returns a string representation of the first day of the current month.
* RML
*/
function stringFirstOfTheMonth($format)
{
    return date($format, firstOfTheMonth(DATENOW));
}

?>