<?php
//=======================================
//###################################
// Kayako Infotech Ltd. - SWIFT Framework
//
// Source Copyright 2001-2008 Kayako Infotech Ltd.
// Unauthorized reproduction is not allowed
// License Number: $%LICENSE%$
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//                   www.kayako.com
//###################################
//=======================================

define("GRID_LIMIT", 1);
define("GRID_SORTORDER", 2);
define("GRID_SORTBY", 3);
define("GRID_TOTALITEMS", 4);

/**
* ###############################################
* CLASS START: Grid
* ###############################################
*/

/*
* Grid class to display data in tabular manner
*/
class Grid
{
	var $gridcache = array();

	/*
	* Core function which generates the entire grid
	*/
	function start($name, $title, $selectquery, $countquery, $searchquery, $fields, $options)
	{
		global $dbCore, $_SWIFT, $settings, $datastore;

		ob_start();

		// Try to load the grid from gridcache
		$staffid = &$_SWIFT["staff"]["staffid"];

		if ($options["nogridload"] == true)
		{
			$_SWIFT["gridcache"][$staffid][$name] = array();
		}
		$_grid = &$_SWIFT["gridcache"][$staffid][$name];
		if (!is_array($_grid))
		{
			// It isnt there, probably the first time its being loaded
			$_SWIFT["gridcache"][$staffid][$name] = array();
		}

		// Is there a limit set in gridcache?
		if (!empty($_REQUEST["set_resultsperpage"]) && is_numeric($_REQUEST["set_resultsperpage"]))
		{
			$_limit = $_REQUEST["set_resultsperpage"];
			$updategrid = true;
		}
		else if (!empty($_grid[GRID_LIMIT]) && is_numeric($_grid[GRID_LIMIT]))
		{
			$_limit = $_grid[GRID_LIMIT];
		} 		
		else
		{
			// Use the default limit
			$_limit = $options["recordsperpage"];
		}
		
		$_grid[GRID_LIMIT] = $_limit;

		// Get the current offset for this page
		if (!empty($_REQUEST["offset"]) && is_numeric($_REQUEST["offset"]))
		{
			$_offset = $_REQUEST["offset"];
		} else {
			$_offset = "0";
		}

		// Is sort order changed or is it default one? First we see if there is a custom sort order passed by user
		if (!empty($_REQUEST["sortby"]))
		{
			$_grid[GRID_SORTBY] = str_replace("-", ".", cleanVar($_REQUEST["sortby"]));
			$_grid[GRID_SORTORDER] = iif(strtolower($_REQUEST["sortorder"])=="desc", "DESC", "ASC");
			$updategrid = true;
		} else if (!empty($_grid[GRID_SORTBY])) {
			// We have a custom sort order already stored in our cache, use that
			$_grid[GRID_SORTBY] = $_grid[GRID_SORTBY];
			$_grid[GRID_SORTORDER] = $_grid[GRID_SORTORDER];
		} else {
			// if all else fails, we just use the default sort order passed to us
			$_grid[GRID_SORTBY] = $options["sortby"];
			$_grid[GRID_SORTORDER] = $options["sortorder"];
		}

		// Build sort statement
		$sortstatement = iif($_grid[GRID_SORTBY], " ORDER BY ". $dbCore->escape($_grid[GRID_SORTBY]) .iif(strtolower($_grid[GRID_SORTORDER])=="desc", " DESC", " ASC"));
		
		/**
		* ADVANCED SEARCH DATA GENERATION
		*/
		for ($ii=0; $ii<count($options["advancedsearch"]); $ii++)
		{
			$selectoptions .= '<option value="'. $ii .'"'. iif($_POST["search_field"]==$ii, " selected") .'>'. $options["advancedsearch"][$ii]["title"] .'</option>'.SWIFT_CRLF;
		}
		
		// ======= FIELD STUFF =======
		for ($ii=0; $ii<count($fields); $ii++)
		{	
			$fielddata .= '<td class="tabletitlerow" width="'.$fields[$ii]["width"].'" align="'. $fields[$ii]["align"] .'" nowrap>';
			if ($_grid[GRID_SORTBY] == $fields[$ii]["name"])
			{
				$fielddata .= '&nbsp;<a href="'. buildURL('index.php?_m='. $_SWIFT["currentmodule"] .'&_a='.$_SWIFT["currentaction"]."&sortby=".urlencode(str_replace(".", "-", $fields[$ii]["name"]))."&sortorder=".iif(strtolower($_grid[GRID_SORTORDER])=="asc", "desc", "asc").$options["appendurl"]).'" '.iif(strtolower($_grid[GRID_SORTORDER])=="asc", 'title="'. $_SWIFT["language"]["sort_desc"] .'"', 'title="'. $_SWIFT["language"]["sort_asc"] .'"').'>'. $fields[$ii]["title"].'&nbsp;<img src="'. $_SWIFT["themepath"] .iif(strtolower($_grid[GRID_SORTORDER])=="asc", "sortasc", "sortdesc").'.gif" border="0" /></a>';
			} else if ($fields[$ii]["type"] == "custom") {
				$fielddata .= '&nbsp;'.$fields[$ii]["title"];
				$norenderoptions = true;
			} else {
				$fielddata .= '&nbsp;<a href="'. buildURL('index.php?_m='. $_SWIFT["currentmodule"] .'&_a='.$_SWIFT["currentaction"]."&sortby=".urlencode(str_replace(".", "-", $fields[$ii]["name"]))."&sortorder=asc".$options["appendurl"]).'" title="'. $_SWIFT["language"]["sort_asc"] .'">'.$fields[$ii]["title"]."</a>";				
			}

			$row .= '<td colspan="'. $fields[$ii]["colspan"] .'" width="'. $fields[$ii]["width"] .'" align="'. $fields[$ii]["align"] .'" valign="'. $fields[$ii]["valign"] .'">['. $fields[$ii]["name"] .']</td>'.SWIFT_CRLF;
			$fielddata .= '</td>'.SWIFT_CRLF;
		}
		// ======= FIELD STUFF =======

		// We need to work out mass action callbacks right from start, we move the option generation to top to save on loops, dont need multiple loops here
		for ($ii=0; $ii<count($options["massaction"]); $ii++)
		{
			if (md5($options["massaction"][$ii]["title"]) == $_POST["mass_action"])
			{
				// We have a match, do a callback
				call_user_func($options["massaction"][$ii]["callback"]);
			}
			$maction .= '<option value="'. md5($options["massaction"][$ii]["title"]) .'">'. $options["massaction"][$ii]["title"] .'</option>'.SWIFT_CRLF;
		}

		// See if we need to do a search
		if (trim($_POST["search_query"]) != "" && trim($_POST["search_field"]) != "")
		{
			$_advancedsearch = true;
		} else {
			$_advancedsearch = false;
		}

		if ((trim($_POST["quicksearch"]) != "" && $options["quicksearch"] == true) || $_advancedsearch == true)
		{
			$_grid[GRID_TOTALITEMS] = "0";
		} else {
			// Lets first get the total
			$dbCore->query($countquery, 2);
			$dbCore->nextRecord(2);
			if (empty($dbCore->Record2["totalitems"]))
			{
				$_grid[GRID_TOTALITEMS] = "0";
			} else {
				$_grid[GRID_TOTALITEMS] = $dbCore->Record2["totalitems"];
			}
		}

		if (is_array($options["massaction"]))
		{
			$_massaction = true;
			$displaytype = 'massaction';
		} else {
			$_massaction = false;
			$displaytype = 'search';
		}

		printFormStart($name);
		echo '<table width="100%" border="0" cellspacing="0" cellpadding="0">'.SWIFT_CRLF;
		
		$gridurl = buildURL("index.php?_m=". $_SWIFT["currentmodule"] ."&_a=". $_SWIFT["currentaction"].$options["appendurl"]);
		// ======= START PAGINATION TABLE =======
		
		$pagination = pagination($gridurl, $_grid[GRID_TOTALITEMS], $_limit, $_offset, "5", "pageoftotal", TRUE, FALSE, TRUE);

		echo '<tr><td><table width="100%"  border="0" cellspacing="0" cellpadding="0"><tr valign="top"><td align="left">'. iif($pagination, '<table border="0" cellpadding="0" cellspacing="1" class="tborder"><tr>'.$pagination.'</tr></table>', '&nbsp;'). '</td>'. iif($options["quicksearch"], '<td width="100%" align="right" valign="middle" width="1"><input type="text" name="quicksearch" class="quicksearch" /></td><td width="1"><img src="'. $_SWIFT["themepath"] .'space.gif" width="4" height="1" /></td>').'<td align="right" width="1"><table border="0" cellpadding="0" cellspacing="1" class="tborder"><tr>'. iif($options["quicksearch"], '<td class="navpageselected" nowrap><a onclick="javascript:document.'.$name.'.submit();" href="#" title="'. $_SWIFT["language"]["quicksearch"] .'">'. $_SWIFT["language"]["quicksearch"] .'</a></td>').'<td class="navpageselected" nowrap><a onclick="javascript:displayGridTabData(\''. $name .'\', false);hideTabOn(\'gridopt'.$name.'\', \''. $displaytype .'\');" href="#" title="'. $_SWIFT["language"]["options"] .'">'. $_SWIFT["language"]["options"] .'</a></td></tr></table></td></tr></table></tr></td>'.SWIFT_CRLF;
		echo '<tr><td><img src="'. $_SWIFT["themepath"] .'space.gif" width="1" height="6" /></tr></td>'.SWIFT_CRLF;
		// ======= END PAGINATION TABLE =======

		echo '<tr><td>'.SWIFT_CRLF;


		$totalfields = iif($_massaction, count($fields)+1, count($fields));

		// ======= START GRID HEADER =======




		// ======= OPTIONS TABS =======
		echo '<table cellspacing="0" cellpadding="0" border="0" width="100%" style="border-collapse: collapse;width: 100%; height: 100%;DISPLAY: none;" id="gridtableopt'. $name .'"><tr style="height: 1em"><td align="left"><div id="gridopt'. $name .'"><ul id="tab">'.iif($_massaction, '<li><a class="currenttab" href="#" onClick="this.blur(); return switchGridTab(\'massaction\', \'gridoptions\')" id="massaction" title="'. $_SWIFT["language"]["mass_action"] .'">'. $_SWIFT["language"]["mass_action"] .'</a></li>').'<li><a href="#" onClick="this.blur(); return switchGridTab(\'search\', \'gridoptions\')" id="search" title="'. $_SWIFT["language"]["advanced_search"] .'">'. $_SWIFT["language"]["advanced_search"] .'</a></li><li><a href="#" onClick="this.blur(); return switchGridTab(\'settings\', \'gridoptions\');" id="settings" title="'. $_SWIFT["language"]["settings"] .'">'. $_SWIFT["language"]["settings"] .'</a></li></ul></div></td></tr>'.SWIFT_CRLF;

		// ======= MASS ACTION =======
		echo '<tr style="height: 1em"><td align="left"><div id="tab_massaction" style="DISPLAY: none;" class="tabcontent"><table border="0" cellpadding="3" cellspacing="1" width="100%" class="tborder"><tr class="row1"><td align="left" width="60">'. $_SWIFT["language"]["massfieldaction"] .'</td><td align="left"><select name="mass_action" onChange="javascript:return doFormConfirm(\''. iif($options["confirmmsg"] == "", $_SWIFT["language"]["actionconfirm"], $options["confirmmsg"]) .'\', this);" class="swiftselect">';
		echo '<option name="firstselect" value="">'. $_SWIFT["language"]["firstselect"] .'</option>'.SWIFT_CRLF;
		echo $maction;
		echo '</select></td></tr></table><BR /> </div>';
		
		// ======= SEARCH =======
		echo '<div id="tab_search" style="DISPLAY: none;" class="tabcontent"><table border="0" cellpadding="3" cellspacing="1" class="tborder" width="100%"><tr class="row1"><td align="left" width="60">'. $_SWIFT["language"]["searchfieldquery"] .'</td><td align="left"><input type="text" name="search_query" class="swifttext" /></td></tr><tr class="row2"><td align="left" width="60">'. $_SWIFT["language"]["searchfieldfield"] .'</td><td align="left"><select name="search_field" class="swiftselect">'.$selectoptions.'</select></td></tr><tr class="row1"><td align="left" colspan="2"><input type="submit" name="searchbutton" value="'. $_SWIFT["language"]["searchbutton"] .'" class="yellowbuttonbig" /></td></tr></table><BR /> </div>';

		// ======= SETTINGS =======
		echo '<div id="tab_settings" style="DISPLAY: none;" class="tabcontent"><table border="0" cellpadding="3" cellspacing="1" class="tborder" width="100%"><tr class="row1"><td align="left" width="120">'. $_SWIFT["language"]["settingsfieldresultsperpage"] .'</td><td align="left"><input type="text" name="set_resultsperpage" value="'. $_grid[GRID_LIMIT] .'" class="swifttext" size="10" /></td></tr><tr class="row2"><td align="left" colspan="2"><input type="submit" name="searchbutton" value="'. $_SWIFT["language"]["submit"] .'" class="yellowbuttonbig" /></td></tr></table><BR /> </div></td></tr></table>';

		$_istabs = false;
		if (count($options["tabs"]))
		{
			$_istabs = true;
			echo '<table cellspacing="0" cellpadding="0" border="0" width="100%" id="gridtableopt"><tr style="height: 1em"><td align="left"><div id="gridopt"><ul id="tab">'.SWIFT_CRLF;
			foreach ($options["tabs"] as $key=>$val)
			{
				echo '<li><a'. iif($val["selected"]==true, ' class="currenttab"') .' href="'. $val["url"] .'" title="'. $val["title"] .'">'. $val["title"] .'</a></li>';
			}
			echo '</ul></div></td></tr>'.SWIFT_CRLF;
			echo '<tr style="height: 1em"><td align="left"<div id="gridtabcontent" style="DISPLAY: block;" class="tabcontent">'.SWIFT_CRLF;
		}

		echo '<table cellpadding="0" cellspacing="0" border="0" width="100%" class="tborder" style="float: left; position: relative;">'.SWIFT_CRLF;
		if (!$_istabs)
		{
			echo '<thead>
			<tr>
			<td class="tcat" width="1" align="left" nowrap><img src="'. $_SWIFT["themepath"] .'space.gif" width="4" height="21"></td>
			<td class="tcat" width="100%" colspan="" align="left" nowrap>'. $title .'</td>
			</tr>
			</thead>';
		}
		echo '<tbody><tr><td class="contenttableborder" colspan="2"><table border="0" cellpadding="3" cellspacing="1" width="100%">'.SWIFT_CRLF;

		// ======= TOOLBAR OPTIONS =======
		if ($_massaction)
		{
			$colspan = count($fields)+1;
		} else {
			$colspan = count($fields);
		}

		if (count($options["toolbar"]))
		{
			echo '<tr>'.SWIFT_CRLF;
			echo '<td class="toolbarrow" align="left" colspan="'. $colspan .'" valign="middle" nowrap><table width="100%" border="0" cellspacing="0" cellpadding="0"><tr><td width="5"><img src="'. $_SWIFT["themepath"] .'grip.gif" border="0" /></td><td class="toolbarrow"><span class="toolbartext">&nbsp;';
			
			for ($ii=0; $ii<count($options["toolbar"]); $ii++)
			{
				echo '<a href="'. $options["toolbar"][$ii]["url"] .'" id="white" title="'. $options["toolbar"][$ii]["title"] .'">'. $options["toolbar"][$ii]["title"] .'</a>';
				if ($ii != (count($options["toolbar"])-1))
				{
					echo " | ";
				}
			}

			echo '</span></td></tr></table></td>'.SWIFT_CRLF;
			echo '</tr>'.SWIFT_CRLF;
		}
		// ======= TOOLBAR OPTIONS =======

		echo '<tr>'.SWIFT_CRLF;
		if ($_massaction)
		{
			echo '<td class="tabletitlerow" align="center" valign="middle" width=20 nowrap><input type="checkbox" name="allselect" class="swiftcheckbox" onClick="javascript:toggleAll(document.'.$name.');if (this.checked) { displayGridTabData(\''. $name .'\', true);hideTabOn(\'gridopt'.$name.'\', \''. $displaytype .'\'); }"></td>'.SWIFT_CRLF;
		}
		
		echo $fielddata;
		unset($fielddata);

		echo '</tr>'.SWIFT_CRLF;
		// ======= END GRID HEADER =======


		// ======= START GRID CONTENTS =======
		// Fetch the required data now
		if (trim($_POST["quicksearch"]) != "" && $options["quicksearch"] == true)
		{
			$_searchstr = $dbCore->escape($_POST["quicksearch"]);
			$_sortjoin = $sortstatement;
			eval("\$queryexec = \"". $searchquery ."\";");
		} else if ($_advancedsearch == true && isset($options["advancedsearch"][$_POST["search_field"]])) {
			$_searchstr = $dbCore->escape($_POST["search_query"]);
			$_sortjoin = $sortstatement;
			eval("\$queryexec = \"". $options["advancedsearch"][$_POST["search_field"]]["query"] ."\";");
		} else {
			$_sortjoin = $sortstatement." LIMIT ". $dbCore->escape($_offset) .", ". $dbCore->escape($_limit) .";";
			eval("\$queryexec = \"". $selectquery ."\";");
		}

		$dbCore->query($queryexec, 2);

		$cnt = 0;
		while ($dbCore->nextRecord(2))
		{
			$cnt++;
			$rowbg = getRowBG();
			if (!$options["idname"])
			{
				$id = $cnt;
			} else {
				$id = $dbCore->Record2[$options["idname"]];
			}

			echo '<tr id="trid'. $id .'" class="'. $rowbg .'" onmouseover="'. "showHighlight(this);" .'" onmouseout="'. "clearHighlight(this, '$rowbg');" .'" onclick="'. "if (document.{$name}.itemhighlight{$id}.value == '1') { this.className = 'rowselect';} else { this.className = '$rowbg';}" .'">'.SWIFT_CRLF;
			if ($_massaction)
			{
				echo '<td align="center" valign="middle">'. "<input type='checkbox' name='itemid[]' value='$id' class=\"swiftcheckbox\" onclick=\"if (document.{$name}.itemhighlight{$id}.value == '0') {document.{$name}.itemhighlight{$id}.value = '1';} else {document.{$name}.itemhighlight{$id}.value = '0';  } if (this.checked) { displayGridTabData('". $name ."', true);hideTabOn('gridopt".$name."', '". $displaytype ."'); }\"><input type='hidden' name='itemhighlight{$id}' value='0'>" .'</td>'.SWIFT_CRLF;
			}

			// We replace all instances of field names in [] braces with their respective values, *much* faster (trust me) than doing a for loop for fields inside another while loop ;)
			
			if ($options["callback"])
			{
				$rowresult = call_user_func($options["callback"], array($dbCore->Record2, $fields));
			} else {
				$rowresult = $dbCore->Record2;
			}
			echo preg_replace("/\[(.*)\]/e", "\$rowresult['\\1']", $row);

			echo '</tr>'.SWIFT_CRLF;
		}

		printMainTableFooter();
		if ($_istabs == true)
		{
			echo '</div></td></tr></table>';
		}

		// ======= START SECOND PAGINATION TABLE =======
		echo '<tr><td><img src="'. $_SWIFT["themepath"] .'space.gif" width="1" height="6" /></tr></td>'.SWIFT_CRLF;
		echo '<tr><td><table width="100%"  border="0" cellspacing="0" cellpadding="0"><tr valign="top"><td align="left"><table border="0" cellpadding="0" cellspacing="1" class="tborder"><tr>'.$pagination.'</tr></table></td></tr></table></tr></td>'.SWIFT_CRLF;
		// ======= END SECOND PAGINATION TABLE =======

		echo '</tr></td></table>'.SWIFT_CRLF;
		// ======= END GRID CONTENTS =======
		

		addHiddenField("_m", $_SWIFT["currentmodule"]);
		addHiddenField("_a", $_SWIFT["currentaction"]);
		if (count($options["hiddenfields"]))
		{
			foreach ($options["hiddenfields"] as $optkey=>$optval)
			{
				addHiddenField($optkey, $optval);
			}
		}
		printFormEnd();

		// Do we need to store any setting changes?
		if ($updategrid == true)
		{
			$datastore->update("gridcache", $_SWIFT["gridcache"]);
		}

		$this->gridcache[$name] = ob_get_contents();
		ob_end_clean();

	}

	/**
	* Displays the last generated grid
	*/
	function display($name)
	{
		echo $this->gridcache[$name];
	}
}

/**
* ###############################################
* CLASS END: Grid
* ###############################################
*/




/**
* ###############################################
* ALL FORM FRAMEWORK FUNCTION DECLARATIONS
* ###############################################
*/

/*
* string pagination(string base_url, int total_items, int items_per_page, int start_item,
* int page_range, mixed pages_text, mixed prevnext_links, bool prevnext_always,
* mixed firstlast_links, bool firstlast_always);
* 
* Used for:
* Creating a pagination link set, a pagination is used to divide a large list of items
* into several pages. This function will help you link to those pages. It is important
* that you use a good routine for this troughout your website, as it can get
* frustrating to have to do this seperately on each and every page you create.
* 
* This function won't create the pages, you will have to do that yourself, but it will
* provide you with a method to easily link to them, with per page customisable ranges
* and looks.
* 
* The pagination can look like or be similar to this:
* 10 Pages  < 2 3 [4] 5 6 > 
* 
* As you can see it does not display all the pages, but only a certain range, this range
* can be set by you, just like various other options, which are explained below.
* Parameters explained:
* - base_url : The links will refer to this url, don't include arguments
* - total_items : The total of items for the list
* - items_per_page : Number of items to show per page
* - start_item : Item to start at (start counting at 0, not 1!)
* - page_range : The range of pages to show, can be 1 to infinite
* - pages_text : The type for the page text to the left of the pagination:
* - total : Only shows total pages
* - page : Only shows current page
* - pageoftotal : Shows current page and total
* - FALSE (bool) : Shows nothing
* - prevnext_links : The type for the previous and next links:
* - num : Shows number for previous and next page
* - nump : Only shows number for previous page
* - numn : Only shows number for next page
* - TRUE (bool) : Show, but no numbers
* - FALSE (bool) : Shows nothing
* - prevnext_always : Always show previous and next links, depending on prevnext_links
* - firstlast_links : The type for the first and last links:
* - num : Shows number for first and last page
* - numf : Only shows number for first page
* - numl : Only shows number for last page
* - TRUE (bool) : Show, but no numbers
* - FALSE (bool) : Show nothing
* - firstlast_always : Always show first and last links, depending on firstlast_links
*/
function pagination($url, $total_items, $per_page, $start, $range = 5, $pages = 'total', $prevnext = TRUE, $prevnext_always = FALSE, $firstlast = TRUE, $firstlast_always = FALSE, $viewall = false) { 
    // Added by Varun
	global $_SWIFT;

	// Here you can edit the looks of the pagination, make sure you keep all instances 
    // of %s intact, they will be replaced by text by the script. Read 
    // http://www.php.net/sprintf for more information on how to change the order of 
    // these format 'tags'. 

    // The links themselves, for these goes: first %s is the url, the second the text for the link 
    $str_links = "<td class='navpage'><a href='%s' title='". $_SWIFT["language"]["pagination"] ."'>%s</a></td>".SWIFT_CRLF; 
    $str_selected = "<td class='navpageselected'><a href='%s' title='". $_SWIFT["language"]["pagination"] ."'>%s</a></td>".SWIFT_CRLF; 
    $str_prevnext = "<td class='navpage'><a href='%s'>%s</a></td>".SWIFT_CRLF;  
    $str_firstlast = "<td class='navpage'><a href='%s'>%s</a></td>".SWIFT_CRLF; 

    // The pages text, has only one %s: the text 
    $str_pages = "%s "; 

    // The text on previous, next, first, and last links. One %s: a (optional) number 
    $prev_txt = '&lt;%s'; 
    $next_txt = '%s&gt;'; 
    $first_txt = '&laquo; '.$_SWIFT["language"]["first"].'<!--%s -->'; 
    $last_txt = '<!-- %s -->'.$_SWIFT["language"]["last"].' &raquo;'; 

    // Pretty self explanatory now.. 
    $pages_txt_total = '%s'; 
    $pages_txt_page = 'Page %s'; 
    $pages_txt_pageoftotal = '<td class="highlightpage">'.iif($_REQUEST["viewall"]==1, $_SWIFT["language"]["allpages"], $_SWIFT["language"]["pagination"]).'</td>'.SWIFT_CRLF; 


    /*******************************Start of the code**************************/ 

    $str = ''; 

    // First, check on a few parameters to see if they're ok, we don't want negatives 
    $total_items = ($total_items < 0) ? 0 : $total_items; 
    $per_page = ($per_page < 1) ? 1 : $per_page; 
    $range = ($range < 1) ? 1 : $range; 
    $sel_page = 1; 

    // Remove the start argument from the url, if it's there, then add the arguments to the url 
    //$args = (isset($_SERVER['argv'][0])) ? preg_replace('/(start=)(\d+)(&|)/i', '', $_SERVER['argv'][0]) : ''; 
    if (substr($url, -4) == '.php')
    {
    	$url .= '?offset=';
    } else {
		$url .= '&offset=';
    }

    $total_pages = ceil($total_items / $per_page); 

    // Are there more than one pages to show? If not, this section will be skipped, 
    // and only the pages_text will be shown 
    if ($total_pages > 1) { 
        // The page we are on 
        $sel_page = floor($start / $per_page) + 1; 

        // The ranges indicate how many pages should be displayed before and after 
        // the selected one. Here, it will check if the range is an even number, 
        // and adjust the ranges appropriately. It will behave best on non-even numbers 
        $range_min = ($range % 2 == 0) ? ($range / 2) - 1 : ($range - 1) / 2; 
        $range_max = ($range % 2 == 0) ? $range_min + 1 : $range_min; 
        $page_min = $sel_page - $range_min; 
        $page_max = $sel_page + $range_max; 

        // This parts checks whether the ranges are 'out of bounds'. If we're at or near 
        // the 'edge' of the pagination, we will start or end there, not at the range 
        $page_min = ($page_min < 1) ? 1 : $page_min; 
        $page_max = ($page_max < ($page_min + $range - 1)) ? $page_min + $range - 1 : $page_max; 
        if ($page_max > $total_pages) { 
                $page_min = ($page_min > 1) ? $total_pages - $range + 1 : 1; 
            $page_max = $total_pages; 
        } 


        // Build the links 
        for ($i = $page_min;$i <= $page_max;$i++) {
			if ($i != 0)
			{
	            $str .= sprintf((($i == $sel_page && empty($_REQUEST["viewall"])) ? $str_selected : $str_links), $url . (($i - 1) * $per_page),$i, $total_pages, $i);
			}
        } 


        // Do we got previous and next links to display? 
        if (($prevnext) || (($prevnext) && ($prevnext_always))) { 
            // Aye we do, set what they will look like 
            $prev_num = (($prevnext === 'num') || ($prevnext === 'nump')) ? $sel_page - 1 : ''; 
            $next_num = (($prevnext === 'num') || ($prevnext === 'numn')) ? $sel_page + 1 : ''; 

            $prev_txt = sprintf($prev_txt, $prev_num); 
            $next_txt = sprintf($next_txt, $next_num); 

            // Display previous link? 
            if (($sel_page > 1) || ($prevnext_always)) { 
                $start_at = ($sel_page - 2) * $per_page; 
                $start_at = ($start_at < 0) ? 0 : $start_at; 
                $str = sprintf($str_prevnext, $url . $start_at, $prev_txt).$str; 
            } 
            // Next link? 
            if (($sel_page < $total_pages) || ($prevnext_always)) { 
                $start_at = $sel_page * $per_page; 
                $start_at = ($start_at >= $total_items) ? $total_items - $per_page : $start_at; 
                $str .= sprintf($str_prevnext, $url . $start_at, $next_txt); 
            } 
        } 

        // This part is just about identical to the prevnext links, just a few minor 
        // value differences 
        if (($firstlast) || (($firstlast) && ($firstlast_always))) { 
            $first_num = 1; 
            $last_num = $total_pages; 

//			$first_num = (($firstlast === 'num') || ($firstlast === 'numf')) ? 1 : ''; 
  //          $last_num = (($firstlast === 'num') || ($firstlast === 'numl')) ? $total_pages : ''; 
            $first_txt = sprintf($first_txt, $first_num); 
            $last_txt = sprintf($last_txt, $last_num); 

            if ((($sel_page > ($range - $range_min)) && ($total_pages > $range)) || ($firstlast_always)) { 
                $str = sprintf($str_firstlast, $url . '0', $first_txt).$str; 
            } 
            if ((($sel_page < ($total_pages - $range_max)) && ($total_pages > $range)) || ($firstlast_always)) { 
				$lastoffset = getLastPageOffset($total_items, $per_page);  // Fixed offset bug [VARUN]
                $str .= sprintf($str_firstlast, $url . $lastoffset, $last_txt); 
            }
        } 
    } 

    // Display pages text? 
    if ($pages) { 
        // Decide what to show 
        switch ($pages) { 
            case 'total': 
                $pages_txt = sprintf($pages_txt_total, $total_pages); 
                break; 
            case 'page': 
                $pages_txt = sprintf($pages_txt_page, $sel_page); 
                break; 
            case 'pageoftotal': 
				if ($total_pages == "0")
				{
					$total_pages = "1";
				}
                $pages_txt = sprintf($pages_txt_pageoftotal, $sel_page, $total_pages); 
                   break; 
        } 
        // Replace it 
        $str = sprintf($str_pages, $pages_txt) . $str; 
    } 

	if ($viewall)
	{
		$str .= '<td class="navpage'. iif($_REQUEST["viewall"]==1, 'selected') .'"><a href="'. $url .'&viewall=1">'.$_SWIFT["language"]["viewall"].'</a></td>'.SWIFT_CRLF; 
	}
    // Done, return the pagination
    return $str; 
}

/**
* Gets the last page offset
*/
function getLastPageOffset($total_items, $per_page)
{
	$total_pages = ceil($total_items / $per_page);
    $lastoffsetpage = floor($total_items / $per_page);
	if ($lastoffsetpage == $total_pages)
	{
		$lastoffsetpage--;	
	}
	$lastoffset = $lastoffsetpage*$per_page;

	return $lastoffset;
}

/*
* #########################################################
* FORM FRAMEWORK FUNCTIONS
* #########################################################
*/

/*
* Returns an HTML Attribute if $data is not empty
*/
function returnHTMLAttrib($attribname, $data)
{
	if ($attribname)
	{
		return " {$attribname}=\"". $data .'"';
	}
}

/*
* Starts a table, You can also pass on $titlearray and it will add a Title, multiple values results in multiple columns
*/
function printTableHeader($titlearray, $width = "100%", $cellpadding = "0", $cellspacing = "0", $tbclass = "tborder", $style = "")
{
	global $_SWIFT;

	echo '<table cellpadding="'. $cellpadding .'" cellspacing="'. $cellspacing .'" border="0" width="'. $width .'" class="'. $tbclass .'" style="'. $style .'">'.SWIFT_CRLF;
	
	if (count($titlearray) > 0)
	{
		// Display header only if needed
		echo '<thead>'.SWIFT_CRLF;
		echo '<tr>'.SWIFT_CRLF;

		if (count($titlearray) > 0)
		{
			echo '<td class="tcat" width="1" align="left" nowrap><img src="'. $_SWIFT["themepath"] .'space.gif" width="4" height="21"></td>'.SWIFT_CRLF;
		}

		for ($ii=0; $ii<count($titlearray); $ii++)
		{
			echo '<td class="tcat" width="'.$titlearray[$ii]["width"].'" colspan="'.$titlearray[$ii]["colspan"].'" align="'. $titlearray[$ii]["align"] .'" nowrap>'. $titlearray[$ii]["title"] .'</td>'.SWIFT_CRLF;
		}

		echo '</tr>'.SWIFT_CRLF;		
		echo '</thead>'.SWIFT_CRLF;
	}
}

/**
* Ends an already started table
*/
function printTableFooter() {
	echo '</table>'.SWIFT_CRLF;
}

/**
* Prints an information box
*/
function printInfoBox($message)
{
	global $_SWIFT;

	if (trim($message) == "")
	{
		return;
	}

	printTableHeader(array(), "100%", "0", "0", "infomsgborder");
	echo '<tbody><tr class="infomsg"><td><table border="0" cellpadding="3" cellspacing="1" width="100%">'.SWIFT_CRLF;
	echo '<tr><td width="16" align="left" valign="middle"><img src="'. $_SWIFT["themepath"] .'icon_info.gif" border="0" /></td>';
	echo '<td align="left"><span class="smalltext">'. $message .'</span></td>';
	echo '</tr>';
	echo '</table></td></tr></tbody></table><BR />'.SWIFT_CRLF;
}

/**
* Prints an error box
*/
function printErrorBox($message)
{
	global $_SWIFT;

	if (trim($message) == "")
	{
		return;
	}

	printTableHeader(array(), "100%", "0", "0", "tbordererror");
	echo '<tbody><tr class="errorbox"><td><table border="0" cellpadding="3" cellspacing="1" width="100%">'.SWIFT_CRLF;
	echo '<tr><td width="16" align="left" valign="middle"><img src="'. $_SWIFT["themepath"] .'icon_error.gif" border="0" /></td>';
	echo '<td align="left" class="errorbox"><strong>'. $message .'</strong></td>';
	echo '</tr>';
	echo '</table></td></tr></tbody></table><BR />'.SWIFT_CRLF;
}

/**
* Main Table Header
*/
function printMainTableHeader($title, $width = "100%", $cellpadding = "0", $cellspacing = "0", $style = "")
{
	$tabletitle[0]["width"] = "100%";
	$tabletitle[0]["align"] = "left";
	$tabletitle[0]["title"] = $title;

	printTableHeader($tabletitle, $width, $cellpadding, $cellspacing, "tborder", $style);
	echo '<tbody><tr><td class="contenttableborder" colspan="'. (count($tabletitle)+1) .'"><table border="0" cellpadding="3" cellspacing="1" width="100%">'.SWIFT_CRLF;
}

/**
* Prints the main table footer
*/
function printMainTableFooter()
{
	echo '</table></td></tr></tbody></table>'.SWIFT_CRLF;
}

/**
* Starts a Form Tag, Dont forget to call FormEnd later on because all hidden fields are added before the form ends
*/
function printFormStart($name = "swiftform", $action = "index.php", $method = "POST", $isupload = false, $target = "", $onsubmit="") {
	global $_SWIFT;

	echo '<form name="'.$name.'" id="'.$name.'" action="'.$action.'"'.iif(!empty($onsubmit),' onsubmit="'. $onsubmit .'"').' method="'.$method.'"'.iif($isupload, ' enctype="multipart/form-data"').iif($target, ' target="'.$target.'"').'>'.SWIFT_CRLF;
}

/**
* Ends an already started form and prints out all pre added hidden fields
*/
function printFormEnd() {
	global $settings, $_SWIFT;
	
	if ($settings->store["security_sessionvar"] == "url" || $settings->store["security_sessionvar"] == "urlcookie")
	{
		printHiddenField("_s", $_SWIFT["sessionid"]);
	}

	printHiddenFields();
	unset($_SWIFT["hiddenfields"]);
	echo '</form>'.SWIFT_CRLF;
}

/**
* Prints a row, A row can have any type of data, this function basically handles all calls for Text, Radio and other Row handling functions
*/
function printDataRow($colarray, $tdclass = "", $title = "", $onmouseover = "", $onmouseout = "", $onclick = "", $id = "", $style = "") {
	if (!empty($tdclass)) {
		$rowbg = $tdclass;
	} else {
		$rowbg = getRowBG();
	}

	echo '<tr class="'. $rowbg .'" title="'. $title .'" onmouseover="'. $onmouseover .'" onmouseout="'. $onmouseout .'" onclick="'. $onclick .'" id="'. $id .'" style="'. $style .'">'.SWIFT_CRLF;
	for ($ii=0; $ii<count($colarray); $ii++) {
		if (empty($colarray[$ii]["align"]))
		{
			$colarray[$ii]["align"] = "center";
		}
		if (empty($colarray[$ii]["valign"]))
		{
			$colarray[$ii]["valign"] = "middle";
		}

		echo '<td class="'. iif(!empty($colarray[$ii]["class"]), $colarray[$ii]["class"], $rowbg) .'" align="'. $colarray[$ii]["align"] .'" valign="'. $colarray[$ii]["valign"] .'" colspan="'. $colarray[$ii]["colspan"] .'" width="'. $colarray[$ii]["width"] .'">'. $colarray[$ii]["value"] .'</td>'.SWIFT_CRLF;
	}
	echo '</tr>'.SWIFT_CRLF;
}

/**
* Prints a Two Column Row
*/
function printDefaultRow($title, $value, $id = "", $style = "") {
	if (empty($title) || empty($value)) {
		return false;
	}

	$colarray[0]["align"] = "left";
	$colarray[0]["valign"] = "top";
	$colarray[0]["nowrap"] = true;
	$colarray[0]["value"] = $title;
	$colarray[0]["width"] = "50%";
	$colarray[1]["align"] = "left";
	$colarray[1]["valign"] = "top";
	$colarray[1]["value"] = $value;

	printDataRow($colarray, "", "", "", "", "", $id, $style);
}

/**
* Prints a Two Column Row
*/
function printDefaultDescRow($title, $description = "", $value, $titleclass = "tabletitle") {
	$info = '<span class="'. $titleclass .'">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');


	$colarray[0]["align"] = "left";
	$colarray[0]["valign"] = "top";
	$colarray[0]["nowrap"] = true;
	$colarray[0]["value"] = $info;
	$colarray[0]["width"] = "50%";
	$colarray[1]["align"] = "left";
	$colarray[1]["valign"] = "middle";
	$colarray[1]["value"] = '<span class="tabledescription">'.$value.'</span>';

	printDataRow($colarray);
}

/**
* Prints a single column description row, This is usefull to display labels and other title information
*/
function printDescRow($description, $colspan = "2", $tdclass = "descrow") {
	if (empty($description)) {
		return false;
	}
	
	$colarray[0]["align"] = "left";
	$colarray[0]["valign"] = "top";
	$colarray[0]["value"] = $description;
	$colarray[0]["colspan"] = $colspan;

	printDataRow($colarray, $tdclass);
}

/**
* Prints a Row with Two Radio buttons, One Yes and Other No
*/
function printYesNoRow($name, $title, $description = "", $isyes = false, $onclick = "", $id = "", $rowstyle = "") {
	global $_SWIFT;
	if (empty($name)) {
		return false;
	}
	if ($_POST[$name] == 1) {
		$isyes = true;
	} else if ($_POST[$name] == "0") {
		$isyes = false;
	}

	$data = '<label for="y'. $name .'">'. '<input type="radio" name="'. $name .'" class="swiftradio" onClick="'. $onclick .'" id="y'. $name .'" value="1"'. iif($isyes, " checked") .' />'.$_SWIFT["language"]["yes"].'</label>'.SWIFT_CRLF;
	$data .= '<label for="n'. $name .'">'. '<input type="radio" name="'. $name .'" onClick="'. $onclick .'" id="n'. $name .'" value="0"'. iif($isyes, "", " checked") .' />'.$_SWIFT["language"]["no"].'</label>'.SWIFT_CRLF;

	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');

	return printDefaultRow($info, $data, $id, $rowstyle);
}

/**
* Prints a Row with Public & Private Radio Buttons
*/
function printPublicPrivateRow($name, $title, $description = "", $ispublic = false) {
	global $_SWIFT;
	if (empty($name)) {
		return false;
	}
	if ($_POST[$name] == 1) {
		$isyes = true;
	}
	$data = '<label for="'. $name .'yes">'. '<input type="radio" name="'. $name .'" class="swiftradio" id="'. $name .'yes" value="1"'. iif($ispublic, " checked") .' />'.$_SWIFT["language"]["public"].'</label>'.SWIFT_CRLF;
	$data .= '<label for="'. $name .'no">'. '<input type="radio" name="'. $name .'" id="'. $name .'no" value="0"'. iif($ispublic, "", " checked") .' />'.$_SWIFT["language"]["private"].'</label>'.SWIFT_CRLF;
	
	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');
	
	return printDefaultRow($info, $data);
}

/**
* Prints a Row to select a color
*/
function printColorRow($name, $title, $description = "", $defaultcolor = "#FFFFFF") {
	global $_SWIFT;
	if (empty($name)) {
		return false;
	}

	if (trim($_POST[$name]) != "") {
		$defaultcolor = $_POST[$name];
	}

	$data = '<table width="100%"  border="0" cellspacing="4" cellpadding="0"><tr><td width="50">'.SWIFT_CRLF;
	$data .= '<input type="text" class="swifttext" name="'. $name .'" id="'. $name .'" size="10" value="'. $defaultcolor .'" onChange="changeColorTable(\''. $name .'\', this.value);" onFocus="changeColorTable(\''. $name .'\', this.value);" onBlur="changeColorTable(\''. $name .'\', this.value);" />'.SWIFT_CRLF;
	$data .= '</td><td align="left"><table width="50"  border="0" cellpadding="1" cellspacing="0" bgcolor="#333333"><tr><td height="16"><table width="100%"  border="0" cellpadding="0" cellspacing="0"><tr><td id="color'. $name .'" bgcolor="'. $defaultcolor .'"><img src="'. $_SWIFT["themepath"] .'space.gif" height="16" /></td></tr></table></td></tr></table></td></tr></table>';

	
	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');
	
	return printDefaultRow($info, $data);
}

/**
* Prints a row with single checkbox button
*/
function printCheckboxRow($name, $title, $description = "", $text = "", $value = 1, $ischecked = false) {

	// A bug currently affects this method. However, this method is not in use. 
	// See bug #638 http://bugs.kayako.com/index.php?cmd=view&id=638

	global $_SWIFT;
	if (empty($name)) {
		return false;
	}
	if ($_POST[$name] == $value) {
		$ischecked = true;
	}

	$title = '<label for="'. $name .'">'. $title .'</label>';
	$data = '<label for="'. $name .'">'. '<input type="checkbox" name="'. $name .'" class="swiftcheckbox" id="'. $name .'" value="'. $name .'"'. iif($ischecked, " checked") .' />'.$text.'</label>'.SWIFT_CRLF;

	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');
	
	return printDefaultRow($info, $data);
}

/**
* Prints a list of checkboxes
*/
function printCheckboxListRow($name, $title, $description = "", $checkboxarray, $style = "") {
	global $_SWIFT;
	if (empty($name)) {
		return false;
	}

	if ($_POST[$name] == 1) {
		$isyes = true;
	}

	$checked = false;
	for ($ii=0; $ii<count($checkboxarray); $ii++)
	{
		$data .= '<label for="'. $name .'['. $ii .']"><input type="checkbox" id="'. $name .'['. $ii .']" name="'. $name .'['. $ii .']" value="'. $checkboxarray[$ii]["value"] .'"';

		if (@in_array($checkboxarray[$ii]["value"], $_POST[$name]) || (!isset($_POST[$name]) && $checkboxarray[$ii]["checked"] == true))
		{
			$data .= ' checked';
			$checked = true;
		}

		$data .= ' /> <span class="smalltext">'. $checkboxarray[$ii]["title"] .'</label></span><BR />'. SWIFT_CRLF;
	}
	
	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');
	
	return printDefaultRow($info, $data, $name, $style);
}

/**
* Print a row for template group
*/
function printTGroupRow($name, $title, $description, $type = false, $tgrouptype = "", $toassignid = "", $style = "")
{
	global $_SWIFT, $dbCore;

	// User Group Restriction
	$options = array();
	$ii = 0;
	$_checked = false;
	$dbCore->query("SELECT * FROM `". TABLE_PREFIX ."templategroups` ORDER BY `title` ASC;");
	while ($dbCore->nextRecord())
	{
		$_value = $dbCore->Record["tgroupid"];
		$_title = $dbCore->Record["title"];
		if ($type == EDIT)
		{
			if (is_array($_SWIFT["tgroupassigncache"]) && in_array($dbCore->Record["tgroupid"], $_SWIFT["tgroupassigncache"][$tgrouptype][$toassignid]))
			{
				$_checked = true;
			} else {
				$_checked = false;
			}
		} else {
			// Always checked in case of insert
			$_checked = true;
		}

		$data .= '<label for="'. $name .'['. $ii .']"><input type="checkbox" id="'. $name .'['. $ii .']" name="'. $name .'['. $ii .']" value="'. $_value .'"';

		if (@in_array($_value, $_POST[$name]) || (!isset($_POST[$name]) && $_checked == true))
		{
			$data .= ' checked';
		}

		$mod = $ii%2;

		$data .= ' /> <span class="smalltext">'. $_title .'</label></span>'.iif($mod==1, "<BR />"). SWIFT_CRLF;

		$ii++;
	}

	$finaldata = '<div id="tglist" style="display: none;">'.$data.'<BR /><BR /></div><div id="tcustomize"><a href="#" onClick="javascript:switchDisplay(\'tglist\');"><span class="smalltext"><img src="'. $_SWIFT["themepath"] .'icon_customize.gif" border="0" align="absmiddle" />&nbsp;'.$_SWIFT["language"]["tcustomize"].'</span></a></div>';

	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');
	
	return printDefaultRow($info, $finaldata, $name, $style);
}

/**
* Prints a row with a text/password row, You will need to pass on $type as password if you want it set as password
*/
function printTextRow($name, $title, $description = "", $type = "text", $value = "", $fieldsize = "30", $maxlength = 0, $id = "", $rowstyle = "") {
	global $_SWIFT;
	if (empty($name)) {
		return false;
	}

	$data = '<input type="'. $type .'" class="swifttext" name="'. $name .'" id="'. $name .'" value="'. htmlentities($value, ENT_COMPAT, $_SWIFT['language']['charset']) .'" size="'. $fieldsize .'"'. iif($maxlength, ' maxlength="'. $maxlength .'"').' />'.SWIFT_CRLF;

	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');

	return printDefaultRow($info, $data, $id, $rowstyle);
}

/**
* Shortcut to print a number row, calls printTextRow in end
*/
function printNumberRow($name, $title, $description = "", $value = "") {
	
	return printTextRow($name, $title, $description, "text", $value, "10");
}

/**
* Adds a global hidden field for a form, usefull to pass on session ids etc
*/
function addHiddenField($name, $value) {
	global $_SWIFT;
	if (!empty($name) && !empty($value)) {
		$tcount = count($_SWIFT["hiddenfields"]);
		$_SWIFT["hiddenfields"][$tcount]["name"] = $name;
		$_SWIFT["hiddenfields"][$tcount]["value"] = $value;
		return true;
	} else {
		return false;
	}
}

/**
* Returns the globally added hidden fields
*/
function printHiddenFields() {
	global $_SWIFT;

	for ($ii=0; $ii<count($_SWIFT["hiddenfields"]); $ii++) {
		echo '<input type="hidden" name="'. $_SWIFT["hiddenfields"][$ii]["name"] .'" value="'. swifthtmlspecialchars($_SWIFT["hiddenfields"][$ii]["value"]) .'"/>'.SWIFT_CRLF;
	}
	echo SWIFT_CRLF;
}

/**
* Prints a hidden field, needed for hidden fields specific to one form (Modules, Action etc)
*/
function printHiddenField($name, $value) {
	global $_SWIFT;

	echo '<input type="hidden" name="'. $name .'" value="'. swifthtmlspecialchars($value) .'"/>'.SWIFT_CRLF;
}

/**
* Prints a row with single submit button, if reset is passed on then a reset button is displayed too
*/
function printSubmitRow($submitname = "", $colspan = "2", $reset = 0) {
	global $_SWIFT;

	if (empty($submitname)) {
		$submitname = $_SWIFT["language"]["submit"];
	}

	if (!empty($reset)) {
		$resetname = $_SWIFT["language"]["reset"];
	}

	if (strlen($submitname) > 0 && strlen($submitname) <= 8)
	{
		$btnclass = "yellowbutton";
	} else if (strlen($submitname) > 8 && strlen($submitname) <= 12) {
		$btnclass = "yellowbuttonbig";
	} else if (strlen($submitname) > 12 && strlen($submitname) < 34) {
		$btnclass = "yellowbuttonbigbig";
	}

	$data[0]["value"] = '<input type="submit" id="submitbutton" name="submitbutton" class="'. $btnclass .'" value="'. $submitname .'" /> '.iif($reset, '<input type="reset" name="resetbutton" class="swiftbutton" value="'. $resetname .'" />').SWIFT_CRLF;
	$data[0]["align"] = "center";
	$data[0]["colspan"] = $colspan;

	return printDataRow($data);
}

/**
* Gets the current global alternate row css class, If $dontupdate is true it returns the next row class but will not change the global values
*/
function getRowBG($dontupdate = false) {
	global $_SWIFT;
	if ($_SWIFT["tablebg"] == "row1") {
		if ($dontupdate == false)
		{
			// Dont update, used in cases where we only need to get the "proposed next" color row
			$_SWIFT["tablebg"] = "row2";
		}
		return "row1";
	} else {
		if ($dontupdate == false)
		{
			// Dont update, used in cases where we only need to get the "proposed next" color row
			$_SWIFT["tablebg"] = "row1";			
		}
		return "row2";
	}
}

/**
* Prints a row with buttons submitted as an array
*/
function printButtonRow($buttonarray, $colspan = "2") {
	global $_SWIFT;

	for ($ii=0; $ii<count($buttonarray); $ii++) {
		$buttondata .= '<input type="submit" class="swiftbutton" value="'. $buttonarray[$ii]["value"] .'" name="'. $buttonarray[$ii]["value"] .'" onclick="javascript:navigateWindow(\''. $buttonarray[$ii]["link"] .'\');" />&nbsp;&nbsp;';
	}

	$data[0]["value"] = $buttondata.SWIFT_CRLF;
	$data[0]["align"] = "center";
	$data[0]["colspan"] = $colspan;

	return printDataRow($data);
}

/**
* Prints an upload file row, remember to start form with enctype set to multipart/formdata
*/
function printUploadRow($name, $title, $description = "", $fieldsize = "25") {
	printTextRow($name, $title, $description, "file", "", $fieldsize);
}

/**
* Prints a calendar select row, only call this function for fields which will require just date
*/
function printDateRow($name, $title, $description, $defaultvalue = "", $dotime = false) {
	global $_SWIFT;
	require_once("./includes/functions_date.php");
	$dateFormat = getCalendarDateFormats($dotime);
	$hour       = $dotime ? ("', '" . $dateFormat["hour"] . "'") : "'";
	
	$data .= '<table border="0" cellpadding="0" cellspacing="0"><tr><td><input type="text" name="'. $name .'" id="'. $name .'" onclick="return showCalendar(\''. $name .'\', \''. $dateFormat["cal"] . $hour . ');" size="12" readonly="1" value="'. $defaultvalue .'" class="swifttext"/></td><td width="2"><img src="'. $_SWIFT["themepath"] .'space.gif" width="2" border="0"/></td><td><img src="'. $_SWIFT["themepath"] .'calendar.gif" id="'. $name .'_tr" onclick="return showCalendar(\''. $name .'\', \''. $dateFormat["cal"] . $hour . ');" style="cursor: pointer;" title="'. $title .'" align="bottom"/></td></tr></table>';
	
	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');

	return printDefaultRow($info, $data);
}

/**
* Prints a row with error information
*/
function printInfoRow($title, $iserror = false, $colspan = "2") {
	$data[0]["value"] = $title.SWIFT_CRLF;
	$data[0]["align"] = "center";
	$data[0]["colspan"] = "2";

	return printDataRow($data, iif($iserror, "rowerror", "rowinfo"), $errortitle);
}

/**
* Prints a text area row
*/
function printTextareaRow($name, $title, $description = "", $value = "", $charwidth = "30", $numlines = "3", $disabled = false, $id = "", $rowstyle = "", $limitLength=false) {

	global $_SWIFT;
	if (empty($name)) {
		return false;
	}
	
	// todo comeback fix how that id is set
	$textareaId = 'textarea'.md5(rand());  // the arg $id is for the <td>, not the textarea
	$limitCode  = ($limitLength)? " onChange=\"limitText('$textareaId', $limitLength)\" onKeyDown=\"limitText('$textareaId', $limitLength)\" onKeyUp=\"limitText('$textareaId', $limitLength)\"" : "";
	
	$data = '<textarea id="' . $textareaId . '" class="swifttextarea" name="'. $name . "\"{$limitCode} cols=\"" . $charwidth .'" rows="'. $numlines .'"'. iif($disabled, " disabled") .'>'. htmlspecialchars($value) .'</textarea>'.SWIFT_CRLF;
	
	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');

	addHiddenField("iscontenthtml", "0");

	return printDefaultRow($info, $data, $id, $rowstyle);
}

/**
* Prints a WYSIWYG htmlarea area row
*/
function printhtmlAreaRow($name, $title, $value = "", $numlines = "20", $iscontenthtml = 1) {
	global $_SWIFT;
	if (empty($name)) {
		return false;
	}

	$data = array();
	$data[0]["value"] = '<table border="0" cellpading="0" cellspacing="0" width="98%"><tr><td align="left"><textarea class="swifttextarea" name="'. $name .'" id="'. $name .'" style="WIDTH:100%;" cols="80" rows="'. $numlines .'">'. htmlspecialchars($value) .'</textarea></td></tr></table>'.SWIFT_CRLF;
	$data[0]["align"] = "left";
	$data[0]["colspan"] = "2";

	addHiddenField("iscontenthtml", $iscontenthtml);
	printDescRow($title);
	printDataRow($data);
}

/**
* Print a list of radio row
*/
function printRadioRow($name, $title, $description = "", $radioarray, $style = "", $nobr = false) {
	global $_SWIFT;
	if (empty($name)) {
		return false;
	}

	if ($_POST[$name] == 1) {
		$isyes = true;
	}

	$checked = false;
	for ($ii=0; $ii<count($radioarray); $ii++)
	{
		$data .= '<label for="'. $name .'['. $ii .']"><input type="radio" id="'. $name .'['.$ii .']" name="'. $name .'" value="'. $radioarray[$ii]["value"] .'"';

		if (($radioarray[$ii]["value"] == $_POST[$name]) || (!isset($_POST[$name]) && $radioarray[$ii]["checked"] == true))
		{
			$data .= ' checked';
			$checked = true;
		}

		$data .= ' /> <span class="smalltext">'. $radioarray[$ii]["title"] .'</label></span>'.iif(!$nobr, '<BR />'). SWIFT_CRLF;
	}
	
	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');
	
	return printDefaultRow($info, $data, $name, $style);

}

/**
* Prints the row with a SELECT Choice Boxes
*/
function printSelectChoiceRow($name, $title, $description = "", $selectoptionsarray, $select2optionsarray, $options1txt = "", $options2txt = "") {
	global $_SWIFT;
	if (empty($name)) {
		return false;
	}

	printDescRow($title);

	if (!empty($description))
	{
		$data = array();
		$data[0]["value"] = '<span class="smalltext">'. $description .'</span>'.SWIFT_CRLF;
		$data[0]["align"] = "left";
		$data[0]["colspan"] = "2";
		printDataRow($data);
	}

	unset($data);
	$data .= '<table border="0" align="center" cellpadding="3" cellspacing="0"><tr><td>'.SWIFT_CRLF;
	$data .= '<select name="left'. $name .'[]" multiple size="10" class="swiftchoiceselect">'.SWIFT_CRLF;
	$selecteddone = false;
	foreach ($selectoptionsarray as $key=>$val)
	{
		$data .= '<option value="'. $val["value"] .'"';
		if ((($_POST["left".$name] == $val["value"]) || ($_POST["left".$name] == "" && $val["selected"] == true)) && $selecteddone == false)
		{
			$data .= ' selected';
			$selecteddone = true;
		}

		$data .= '>'. str_replace(" ", "&nbsp;", htmlspecialchars($val["title"])) .'</option>'.SWIFT_CRLF;
	}
	$data .= $options1txt;
	$data .= '</select>'.SWIFT_CRLF;
	$data .= '</td><td>'.SWIFT_CRLF;
	$data .= '<input type="button" value ="'. $_SWIFT["language"]["choiceadd"] .'" onClick="listMoveItem(\'left'. $name .'[]\', \''. $name .'[]\')" class="swiftchoicebutton"><br><br/>
	<input type="button" value ="'. $_SWIFT["language"]["choicerem"] .'" onClick="listMoveItem(\''. $name .'[]\', \'left'. $name .'[]\')" class="swiftchoicebutton"><br><br/>
	<input type="button" value ="'. $_SWIFT["language"]["choicemup"] .'" onClick="listMoveUpList(\''. $name .'[]\');" class="swiftchoicebutton"><br><br/>
	<input type="button" value ="'. $_SWIFT["language"]["choicemdn"] .'" onClick="listMoveDownList(\''. $name .'[]\');" class="swiftchoicebutton"></td><td>';

	$data .= '<select name="'. $name .'[]" multiple size="10" class="swiftchoiceselect">'.SWIFT_CRLF;

	$selecteddone = false;
	foreach ($select2optionsarray as $key=>$val)
	{
		$data .= '<option value="'. $val["value"] .'"';
		if ((($_POST[$name] == $val["value"]) || ($_POST[$name] == "" && $val["selected"] == true)) && $selecteddone == false)
		{
			$data .= ' selected';
			$selecteddone = true;
		}

		$data .= '>'. str_replace(" ", "&nbsp;", htmlspecialchars($val["title"])) .'</option>'.SWIFT_CRLF;
	}
	$data .= $options2txt;

	$data .= '</select></td></tr></table>'.SWIFT_CRLF;

	$rowdata = array();
	$rowdata[0]["value"] = $data;
	$rowdata[0]["align"] = "left";
	$rowdata[0]["colspan"] = "2";
	printDataRow($rowdata);

	return true;
}

/**
* Prints the row with a SELECT Box
*/
function printSelectRow($name, $title, $description = "", $selectoptionsarray, $onchange = "", $style = "", $_container = '') {
	global $_SWIFT;
	if (empty($name)) {
		return false;
	}

	if ($_POST[$name] == 1) {
		$isyes = true;
	}
	$data = '<select name="'. $name .'" class="swiftselect"'. iif($onchange!="", ' onChange="'. $onchange .'"') .'>';
	$selecteddone = false;
	for ($ii=0; $ii<count($selectoptionsarray); $ii++)
	{
		$data .= '<option value="'. $selectoptionsarray[$ii]["value"] .'"';
		if ((($_POST[$name] == $selectoptionsarray[$ii]["value"]) || ($_POST[$name] == "" && $selectoptionsarray[$ii]["selected"] == true)) && $selecteddone == false)
		{
			$data .= ' selected';
			$selecteddone = true;
		}

		$data .= '>'. str_replace(" ", "&nbsp;", htmlspecialchars($selectoptionsarray[$ii]["title"])) .'</option>'.SWIFT_CRLF;
	}
	$data .= '</select>'.SWIFT_CRLF;
	
	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');
	
	return printDefaultRow($info, iif(!empty($_container), '<div id="'. $_container .'">') . $data . iif(!empty($_container), '</div>'), $name, $style);
}


/**
* Prints a row with select multiple box allowing selection of multiple options
*/
function printSelectMultipleRow($name, $title, $description = "", $selectoptionsarray, $size = 5, $width = "") {
	global $_SWIFT;
	if (empty($name)) {
		return false;
	}
	if ($_POST[$name] == 1) {
		$isyes = true;
	}
	$data = '<select name="'. $name .'[]" class="swiftselect" size="'. $size .'"'. iif(!empty($width), ' style="WIDTH: '. $width .';"') .' multiple>';
	for ($ii=0; $ii<count($selectoptionsarray); $ii++)
	{
		$data .= '<option value="'. $selectoptionsarray[$ii]["value"] .'"';
		if ((@in_array($selectoptionsarray[$ii]["value"], $_POST[$name])) || (!is_array($_POST[$name]) && $selectoptionsarray[$ii]["selected"] == true))
		{
			$data .= ' selected';
		}

		$data .= '>'. str_replace(" ", "&nbsp;", htmlspecialchars($selectoptionsarray[$ii]["title"])) .'</option>'.SWIFT_CRLF;
	}
	$data .= '</select>'.SWIFT_CRLF;
	
	$info = '<span class="tabletitle">'. $title .'</span>'. iif($description, '<BR /><span class="tabledescription">'. $description .'</span>');
	
	return printDefaultRow($info, $data);
}

/*
* Prints a redirect, can be unix or windows style depending upon setting
*/
function printRedirect($message, $url, $printredir = false)
{
	global $_SWIFT, $settings, $template;

	if ($settings->store["general_redirecttype"] == "windows" || $printredir == true) {
		$template->assign("redirecturl", $url);

		if (SWIFT_AREA == SWIFT_CLIENT || SWIFT_AREA == SWIFT_VISITOR)
		{
			echo $template->displayTemplate("redirect");
		} else {
			echo $template->display("redirect.tpl");
		}

		printTableHeader(array(), "100%", "2", "1");
		printInfoRow('<table border="0" cellpading="0" cellspacing="3"><tr><td align="center"><span class="mediumtext">'.$message.'</span></td></tr><tr><td><span class="smalltext"><a href="'. $url .'">'.$_SWIFT["language"]["no_wait"].'</a></span></td></tr></table>');
		printTableFooter();
	} else if ($settings->store["general_redirecttype"] == "unix") {
		header("location: $url");
	}
}

/**
* Prints a select row with all current user groups
*/
function printUserGroupSelectRow($name, $title, $description, $onlyregistered = false)
{
	global $dbCore, $_UGROUPCACHE;

	if (_is_array($_UGROUPCACHE))
	{
		$_options = $_UGROUPCACHE;
	} else {
		$_options = array();
		$index = $firstone = 0;
		$dbCore->query("SELECT `usergroupid`, `title` FROM `". TABLE_PREFIX ."usergroups`". iif($onlyregistered, " WHERE `grouptype` = '". USERGROUP_REGISTERED ."'")." ORDER BY `title` ASC;", 3);
		while ($dbCore->nextRecord(3))
		{
			$_options[$index]["title"] = $dbCore->Record3["title"];
			$_options[$index]["value"] = $dbCore->Record3["usergroupid"];
			if ($firstone == 0)
			{
				$_options[$index]["selected"] = 1;
				$firstone = 1;
			}
			$index++;
		}
		$_UGROUPCACHE = $_options;		
	}

	printSelectRow($name, $title, $description, $_options);
}

/**
* Strips all HTML and style tags from a string.
* NOTE: Purposely ignores certain tags that are explicitly allowed through other settings.
* -- RML
*/
function stripHTMLTags($str)
{
    global $_SWIFT;
    
    // Some HTML tags are allowed, if the setting is enabled.
    $allowed = array();
    
    if (1 == $_SWIFT["settings"]["t_allowhtml"])
    {
        if (preg_match_all('@<([^>]+)>@i', $_SWIFT["settings"]["t_allowabletags"], $matches))
        {
            $allowed = array_merge($allowed, $matches[1]);
        }
    }
    
	// If "strip script tags" is off, add script to the allowed list.
	if ($_SWIFT["settings"]["pr_stripscript"] == 0)
	{
		$allowed[] = "script";
	}
    
    if (count($allowed))
    {
        $strRegex = '@<(?!(?:[\\/]?)(' . implode("|", $allowed) . ')(?:\s|/>|>)[^<>]*?)[^<>]*?>@si';
    }
    else
    {
        $strRegex = '@<(?:[\\/]?)[^<>]*?>@si';
    }
    
    $search = array(
					'@<style[^<]*?/>@si',						// "Empty" style tags
					'@<style(?:.*?)>.*?</style(?:[\s]*?)>@si',	// "Full" style tags
					$strRegex									// All other tags and HTML comments
					);

    return preg_replace($search, '', $str);
}

/**
* Strips script tags from a string (and only script tags)
* -- RML
*/
function stripScriptTags($html)
{
	// Strips "empty" script tags (e.g. <script type="text/javascript" src="foo"/>)
	$html = preg_replace('@<script[^<]*?/>@si', '', $html);
	
	// Strips "full" script tags (e.g. <script type="text/javascript">foo</script>)
	$html = preg_replace('@<script(?:.*?)>.*?</script(?:[\s]*?)>@si', '', $html);
	
	return $html;
}

?>