<?php
//=======================================
//###################################
// Kayako Infotech Ltd. - SWIFT Framework
//
// Source Copyright 2001-2008 Kayako Infotech Ltd.
// Unauthorized reproduction is not allowed
// License Number: $%LICENSE%$
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//                   www.kayako.com
//###################################
//=======================================

require_once ("./includes/MIME/mimeDecode.php");
require_once ("./includes/MIME/RFC822.php");

define ("RECIPIENT_LIMIT", 200);

/**
* Class to Parse MIME Data
*/
class MimeParser
{
	/*
	* Constructor
	*/
	function MimeParser()
	{
		global $decodeMail_output, $decodeMail_inRelated;

		$decodeMail_output = new stdClass;
		$decodeMail_inRelated = false;
	}

	/**
	* Decodes a mail and returns the various parts.
	* Original Function by Richard Heyes, Modified to fix some bugs
	*/
	function decodeMail($input)
	{
		global $decodeMail_output, $_SWIFT;

    // Decoding policy that transfers encoding to the native
    // help desk encoding
    $myPolicy = new MIMEDecodePolicy;
        
        // Use our 3rd party classes to perform MIME decoding
		$Mail_mimeDecode = new Mail_mimeDecode($input, $myPolicy);
		$Mail_RFC822     = new Mail_RFC822;
		$output          = &$decodeMail_output;

        // Set up the optional items for decoding
		$decode_params['input'] = $input;
		$decode_params['include_bodies'] = true;
		$decode_params['decode_bodies']  = true;
		$decode_params['decode_headers'] = true;

    // Perform the actual decoding
		$structure = $Mail_mimeDecode->decode($decode_params);

		// Traverse over the returned decoded structure (e-mail body, multipart chunks)
		// and perform special processing for each one, depending on the MIME type.
		$this->decodeMail_traverseStructure($structure);

		// Sizes
		$output->textSize = strlen(@$output->text);
		$output->htmlSize = strlen(@$output->html);
//		parserDebug("MIME: Text Size: ".$output->textSize.", HTML Size: ".$output->htmlSize);

		// Headers
		$output->headers = $structure->headers;
//		parserDebug("MIME: Got Headers");

		$tofix = array("\(", "\)", "\[", "\]", "\{", "\}");
		$fixres = array("(", ")", "[", "]", "{", "}");

		// From name/email
		if (!empty($output->headers['from'])) {
			$from = $Mail_RFC822->parseAddressList(str_replace($tofix, $fixres, $output->headers['from']), null, null, false);
			$output->fromEmail = sprintf('%s@%s', $from[0]->mailbox, $from[0]->host);
			$output->fromName  = preg_replace('/^"(.*)"$/', '\1', $from[0]->personal);
		}
//		parserDebug("MIME: Parsed From");

		// Reply to
		if (!empty($output->headers['reply-to'])) {
			$replyto = $Mail_RFC822->parseAddressList(str_replace($tofix, $fixres, $output->headers['reply-to']), null, null, false);
			$output->replytoEmail = sprintf('%s@%s', $replyto[0]->mailbox, $replyto[0]->host);
			$output->replytoName = preg_replace('/^"(.*)"$/', '\1', $replyto[0]->personal);
		}
//		parserDebug("MIME: Parsed Reply To");

		// To name/email
		if (!empty($output->headers['to'])) {
			$to = $Mail_RFC822->parseAddressList(str_replace($tofix, $fixres, $output->headers['to']), null, null, false);
			if (is_array($output->headers['to'])) {
				$output->toEmail = $output->headers['to'][0];
			} else {
				$output->toEmail = sprintf('%s@%s', $to[0]->mailbox, $to[0]->host);
			}
			$output->toName = preg_replace('/^"(.*)"$/', '\1', $to[0]->personal);
		}
//		parserDebug("MIME: Parsed To");

		// In-Reply-To
		if (!empty($output->headers['in-reply-to'])) {
			$inreplyto = $Mail_RFC822->parseAddressList(str_replace($tofix, $fixres, $output->headers['in-reply-to']), null, null, false);
			$output->inReplyTo = trim($inreplyto[0]->mailbox);
		}
//		parserDebug("MIME: Parsed In-Reply-To");

		// Return address
		if (!empty($output->headers['reply-to'])) {
			$output->returnAddress = $output->headers['reply-to'];
		} elseif (!empty($output->headers['sender'])) {
			$output->returnAddress = $output->headers['sender'];
		} elseif (!empty($output->headers['from'])) {
			$output->returnAddress = $output->headers['from'];
		} elseif (!empty($output->headers['return-path'])) {
			$output->returnAddress = $output->headers['return-path'];
		}
		if (!empty($output->returnAddress)) {
			$retAddress = $Mail_RFC822->parseAddressList(str_replace($tofix, $fixres, $output->returnAddress), null, null, false);
			$output->returnAddressEmail = sprintf('%s@%s', $retAddress[0]->mailbox, $retAddress[0]->host);
			$output->returnAddressName = preg_replace('/^"(.*)"$/', '\1', $retAddress[0]->personal);
		}
//		parserDebug("MIME: Calculated the Return Address: ".$output->returnAddress);

		// Recipient addresses
		foreach (array('to', 'cc', 'x-rcpt-to') as $header)
		{
			if (!empty($output->headers[$header]) && !is_array($output->headers['to']))
			{
				$to		   = $Mail_RFC822->parseAddressList(str_replace($tofix, $fixres, $output->headers[$header]), null, null, false);
				$loopcount = count($to);
				
				if ($loopcount > RECIPIENT_LIMIT)
				{
					$loopcount = RECIPIENT_LIMIT;
				}
				
				for ($i=0; $i<$loopcount; $i++)
				{
					if ($to[$i])
					{
						$recipientemail = sprintf('%s@%s', $to[$i]->mailbox, $to[$i]->host);
						
						if ($this->isValidEmail($recipientemail))
						{
							$recipients[] = $recipientemail;
						}
					}
				}
			}
			else
			{
				// Most probably this email has multiple to's
				if ($this->isValidEmail($output->headers['to'][0]))
				{
					$recipients[] = $output->headers['to'][0];
				}
			}
		}

    $output->recipientAddresses = $recipients;
    $output->subject = $output->headers["subject"];

    // Mail_mimeDecode now automatically decodes MIME-encoded "words" (subject, from name, to name etc.)
    // to the native encoding of the help desk.  It is no longer necessary to do any further decoding here.
    //
    // Message bodies will still be subject to $_SWIFT["settings"]["pr_conversion"], however.
    // -- RML

		return $output;
	}

	/**
	* Traverses the decoded structure and adds the various
	* parts to the global var.
	*/
	function decodeMail_traverseStructure($structure, $subpart = false, $rfc822_part = false)
	{
		global $decodeMail_output, $decodeMail_inRelated, $_SWIFT;

    $output = &$decodeMail_output;
		$ctype  = strtolower(@$structure->ctype_primary) . '/' . strtolower(@$structure->ctype_secondary);

		if (!$ctype) {
			$ctype = 'text/plain';
		}

		switch ($ctype) {
		  case 'message/feedback-report':
			case 'text/html':
			case 'text/plain':
				if (!empty($structure->disposition) AND $structure->disposition == 'attachment')
				{
				  // Attachment
					$this->decodeMail_addAttachment($structure);
				}
				else
				{
          // Ignore text/html bodies of embedded message/rfc822 parts. The text bodies of those parts
          // are saved as part of the attachment.
          if (!$rfc822_part)
          {
			      // Textual body
			      $this->decodeMail_decodeBody($structure, $structure->body);
			      $var = $ctype == 'text/html' ? 'html' : 'text';
            @$output->$var .= $structure->body;
          }
				}
				break;
			case 'message/rfc822':
      {
        // For message/rfc822 (original message forwarded as an attachment):
        // 1. Check if the part is actually an MHT; this has special handling needs.
        // 2. Save the entire original message/rfc822 as an attachment.
        // 3. Recursively parse the structure looking for attachments and add them when found.

        $this->decodeMail_ParseRFC822Message($structure); // Adds the entire structure of the RFC822 message as an attachment.

        if (strtolower($this->decodeMail_ExtensionFromFileName($structure->ctype_parameters['name'])) == "mht")
        {
          // Don't recursively parse MHT files.
          break;
        }

        $rfc822_part = true;

        // Let execution fall through to the default handler, which recursively parses the sub-parts of the message.
      }
			case 'multipart/report':
			case 'multipart/signed':
			case 'multipart/mixed':
			case 'multipart/alternative':
			case 'multipart/related':
			case 'multipart/digest':
			case 'multipart/parallel':
			case 'multipart/voice-message':			
			case 'message/disposition-notification':
				if (!empty($structure->parts)) {
					if ($ctype == 'multipart/related') {
						$decodeMail_inRelated = true;
					}
					for ($i=0; $i<count($structure->parts); $i++) {
						$this->decodeMail_traverseStructure($structure->parts[$i], true, $rfc822_part);
					}
					$decodeMail_inRelated = false;
				}
				break;
		
			default:
				$this->decodeMail_addAttachment($structure);
		}
	}

  function decodeMail_ParseRFC822Message($structure)
  {
    // Saves a message/rfc822 MIME part as an attachment to a ticket
    global $decodeMail_output;
    $decodeMail_output->attachments[] = array('data' => $structure->raw,
                                              'size' => strlen($structure->raw),
                                              'filename' => $structure->ctype_parameters['name'],
                                              'extension' => $this->decodeMail_ExtensionFromFileName($structure->ctype_parameters['name']),
                                              'contenttype'=> 'message/rfc822');
  }

  function decodeMail_ExtensionFromFileName($filename)
  {
    $matches = array();

    if (preg_match('/^.+\.(\w+)$/', $filename, $matches))
    {
      return $matches[1];
    }

    return '';
  }
  
	/**
	* Private function used by the traverse function (above)
	* to add an attachment to the output object.
	*/
	function decodeMail_addAttachment($structure)
	{
		global $decodeMail_output;

		$output = &$decodeMail_output;

		// Determine filename and extension
		if (!empty($structure->d_parameters['filename'])) {
      $filename  = $structure->d_parameters['filename'];
			$extension = $this->decodeMail_ExtensionFromFileName($filename);
		} else if (!empty($structure->ctype_parameters["name"])) {
      $filename  = $structure->ctype_parameters["name"];
			$extension = $this->decodeMail_ExtensionFromFileName($filename);
		} else {
			$filename  = '';
			$extension = '';
		}
		$ctype  = strtolower(@$structure->ctype_primary) . '/' . strtolower(@$structure->ctype_secondary);
		// Attachment or embedded?
		$var = 'attachments';
		$output->{$var}[] = array('data'        => @$structure->body,
								              'size'        => @strlen($structure->body),
								              'filename'    => $filename,
								              'extension'   => $extension,
								              'contenttype' => $ctype);
	}
	
	/**
	* Decodes an email message body (or multipart chunk) from whatever encoding it is in to the native encoding
	* of the help desk (if the setting is enabled) in the following fashion:
	*
	* 1. If a content-type charset is present, it is used.
	* 2. If a content-type is not present, but an x-notascii is, it is used.
	* 3. If neither are present, an attempt is made to deduce the encoding based on characters found in the message body.
	* 4. The decision to transfer encoding is then passed to MIMEDecodePolicy.
	*/
	function decodeMail_decodeBody($headers, &$body)
	{
    global $_SWIFT;
    $nativeEncoding = $_SWIFT['language']['charset'];
    
    // Get the source encoding from the headers/body
    $fromEncoding = $this->decodeMail_getEncoding($headers, $body);

    // Convert to the native encoding
    $myPolicy = new MIMEDecodePolicy;
    $body     = $myPolicy->convertEncoding($body, $fromEncoding);
	}

    function decodeMail_getEncoding($headers, $body)
    {
	    $fromCharset = "US-ASCII"; // Default to ASCII
	    
	    // Do we have content-type?
	    if (!empty($headers->ctype_parameters['charset']))
	    {
	        $fromCharset = $headers->ctype_parameters['charset'];
	    }
	    else if (!empty($headers->headers['x-notascii']))
	    {
	        // Format of x-notascii: "x-notascii: charset=foo"
            if (preg_match('/^[^=]*=\s*([-a-z0-9]+)$/i', $headers->headers['x-notascii'], $matches))
            {
                $fromCharset = $matches[1];
            }    
	    }
	    else
	    {
	        // Couldn't find a usable charset anywhere in the headers; let's try to
	        // deduce it directly from the message body.
	        $fromCharset = mb_detect_encoding($body);
	    }
	    
	    return $fromCharset;    
    }
    
	/**
	* Validate the Email Address
	*/
	function isValidEmail($email) {
		return (eregi( 
			'^[-!#$%&\'*+\\./0-9=?A-Z^_`{|}~]+'.      // the user name 
			'@'.                                      // the ubiquitous at-sign 
			'([-0-9A-Z]+\.)+' .                       // host, sub-, and domain names 
			'([0-9A-Z]){2,6}$',                       // top-level domain (TLD)
			trim($email)));
	}
}

/**
* Used by Mail_mimeDecode to perform "encoded-word" decoding
*/
class MIMEDecodePolicy
{
    /**
    * Converts str to the required encoding based on the current encoding,
    * which is specified in fromEncoding.
    */
    function convertEncoding($str /* Input string */, $fromEncoding /* e.g. "KOI8-R" */)
    {
        // Do conversion to native encoding:
        // 1. If the desk is UTF-8, always do it.
        // 2. If the desk is not UTF-8, only do it if pr_conversion is enabled.
        // 3. If the encodings are not already the same.
        global $_SWIFT;
        $nativeEncoding = strtoupper($_SWIFT["language"]["charset"]);


        if ("UTF-8" == $nativeEncoding || 1 == $_SWIFT["settings"]["pr_conversion"])
        {
            if ($nativeEncoding != strtoupper($fromEncoding))
            {
              if (extension_loaded("iconv"))
              {
                // iconv (for charsets not supported by mbstring)
                $str = iconv($fromEncoding, $nativeEncoding, $str);
              }
              else
              {
                // mbstring (default)
                $str = mb_convert_encoding($str, $nativeEncoding, $fromEncoding);
              }
            }
        }

        return $str;
    }
}

?>