<?php
//=======================================
//###################################
// Kayako Infotech Ltd. - SWIFT Framework
//
// Source Copyright 2001-2008 Kayako Infotech Ltd.
// Unauthorized reproduction is not allowed
// License Number: $%LICENSE%$
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//                   www.kayako.com
//###################################
//=======================================

if (!defined("INSWIFT")) {
	trigger_error("Unable to process " . htmlentities($_SERVER['PHP_SELF']), E_USER_ERROR);
}

$template->loadLanguageSection("emailparser");

// Lock file definition
define ("PARSER_LOCKFILE", dirname(__FILE__)."/../../files/parser.lockfile");

// Lock file expiration time
// Default is 1200 seconds; this is the value of the cron task interval (10 minutes) times 2.
define ("PARSER_EXPIRY", 1200);

/**
* Function to clear lock file
*/
function clearParserLockFile()
{
    if (file_exists(PARSER_LOCKFILE))
    {
	    if (!unlink(PARSER_LOCKFILE))
	    {
	        echo "Error: Failed to delete parser lock file (" . PARSER_LOCKFILE . "); error: " . errno . "<br/>";
	    }
	}
}

/**
* Determines if the lock file has expired
*/
function hasParserLockFileExpired()
{
    $ret = false;
    
    if (file_exists(PARSER_LOCKFILE))
    {
        $fp = fopen(PARSER_LOCKFILE, "r");
        
        if ($fp !== false)
        {
            clearstatcache();
            $fileBytes = filesize(PARSER_LOCKFILE);
            $fileData  = fread($fp, $fileBytes);
            
            if ($fileData !== false)
            {
                $elapsed = (DATENOW - intval($fileData));
             
                if ($elapsed >= PARSER_EXPIRY)
                {
                    $ret = true;
                }
            }
            
            fclose($fp);
        }
    }

    return $ret;
}

/**
* Handles shutdown of this script
*/
function parserShutDown()
{
    clearParserLockFile();
}

// We require IMAP.  If it's not installed we can't fetch email through this script anyways.
if (!extension_loaded('imap'))
{
	echo "Error: Your PHP was not compiled with IMAP support.<br/> *nix users should recompile their PHP with the '--with-imap' flag; Windows users can simply uncomment the extension='php_imap.dll' line in their php.ini<br/>";
	clearParserLockFile();
	exit;
}

// This is an argument provided to delete the lock file before processing
if ($_REQUEST['deletelockfile'] == '1')
{
	clearParserLockFile();
}

// Check to see if the lock file is expired.
if (hasParserLockFileExpired())
{
    // It's expired, so delete it.
    clearParserLockFile();
    echo $_SWIFT['language']['lockfileexpired'] . "<br/>";
}

// If the lock file exists, some other instance of cron is still running, so we just bail.
if (file_exists(PARSER_LOCKFILE))
{
	echo sprintf($_SWIFT['language']['errorlockfilefound'], $_SWIFT['swiftpath'].'cron/index.php?_t=parser&deletelockfile=1');
	exit;
}

// Open up the lock file and write the current time in it
$fp = fopen(PARSER_LOCKFILE, "w+");
fwrite($fp, DATENOW);
fclose($fp);

// Make sure the lock file is deleted when we're done executing this pass.
register_shutdown_function("parserShutDown");

$template->loadLanguageSection("staff_tickets");
$template->loadLanguageSection("tickets_auditlogs");

require_once ("./includes/functions_mime.php");
require_once ("./includes/functions_users.php");
require_once ("./modules/parser/functions_parsercore.php");
require_once ("./modules/parser/functions_parserrules.php");
require_once ("./modules/parser/functions_parser_loopcontrol.php");

if (!count($_SWIFT["queuecache"]))
{
	echo "ERROR: No e-mail queues found!  You must configure at least one e-mail queue in order to use the parser.<br/>";
	parserShutDown();
	exit;
}

if ($_GET["debug"] == "1")
{
	$_SWIFT["clidebug"] = true;
	$_SWIFT["dbgfilename"] = "crondbg_".buildHash();
}

define("INPARSER", "1");

$_SWIFT["startparser"] = fetchmicrotime();

if ($module->isRegistered(MODULE_TICKETS))
{
	require_once ("./modules/tickets/functions_ticketcore.php");
	require_once ("./modules/tickets/functions_ticketmain.php");
	require_once ("./modules/tickets/functions_alertrules.php");
	require_once ("./modules/tickets/functions_attachments.php");
	require_once ("./modules/tickets/functions_search.php");
	require_once ("./modules/tickets/functions_sla.php");
} else {
	echo "ERROR: The tickets module is not registered.<br/>";
	parserShutDown();
	exit;
}

$allQueuesAccepted = 0;
$allQueuesRejected = 0;

$dbCore->query("SELECT * FROM `". TABLE_PREFIX ."emailqueues` ORDER BY `emailqueueid` ASC;", 4);
while ($dbCore->nextRecord(4))
{
    if ("pipe" == $dbCore->Record4["fetchtype"])
    {
		continue;  // Skip pipe queues as they're piped in through CLI.
    }
    
    $ispop3 = false;
    
	switch ($dbCore->Record4["fetchtype"])
	{
		case "imap":
		{
			$fetchtype = "imap/notls";
		}
		break;	
		case "imapssl":
		{
			$fetchtype = "imap/ssl/novalidate-cert/notls";
		}
		break;
		case "imaptls":
		{
			$fetchtype = "imap/ssl/novalidate-cert";
		}
		break;
		case "pop3ssl":
		{
			$fetchtype = "pop3/ssl/novalidate-cert/notls";
			$ispop3    = true;
		}
		break;
		case "pop3tls":
		{
			$fetchtype = "pop3/ssl/novalidate-cert";
			$ispop3    = true;
		}
		break;
		default:
		{
			$fetchtype = "pop3/notls";
			$ispop3    = true;
		}
		break;
	}

	echo "Running parser for Queue ID '" . $dbCore->Record4["emailqueueid"] . "' ...<BR />";

	$rejected	= 0;
	$accepted	= 0;
	$totalmsg	= 0;
	$_account	= $dbCore->Record4;
	$mbox		= @imap_open("{". $dbCore->Record4["host"].":".$dbCore->Record4["port"] ."/". $fetchtype ."}INBOX", $dbCore->Record4["username"], $dbCore->Record4["userpassword"]);
	
	if ($mbox)
	{
		$totalmsg = imap_num_msg($mbox);
		$maxmsgs  = intval($_SWIFT["settings"]["pr_procno"]);
		$_uidtype = FT_UID;
		
		if (0 < $totalmsg)
		{
			echo "There are {$totalmsg} messages in the inbox; retrieving...<BR />";
		}
		else
		{
			echo "There are no messages in the inbox; done!<BR />";
		}
		
		@set_time_limit(0);
		
		for ($i = 1; $i <= $totalmsg; ++$i)
		{
			if ($accepted > $maxmsgs)
			{
				// There is a limit on how many messages we are to process at once and we have reached it; bail.
				echo "Reached maximum message count of {$maxmsgs}; quitting.<BR />";
				break;
			}
			
			if ($ispop3)
			{
				$_msguid = $i;
			} else {
				$_msguid = imap_uid($mbox, $i);
			}
			
			echo "{$_msguid}: Downloading message...<BR />";
						
			$structd = imap_fetchstructure($mbox, $_msguid, $_uidtype);
			$bytes   = $structd->bytes;
				
			if (isset($structd->parts) && is_array($structd->parts))
			{
				$bytes += getMessageSize($structd->parts);
			}
		
			$kb = ($bytes / 1024);
			
			if ($kb > $_SWIFT["settings"]["pr_sizelimit"])
			{
				echo "{$_msguid}: Size of " . sprintf("%.02f", $kb) . " KiB exceeds the size limit setting; ignoring...<BR />";
				$rejected++;
				$allQueuesRejected++;
				continue;
			}
			else
			{
                echo "{$_msguid}: Size of " . sprintf("%.02f", $kb) . " KiB is acceptable...<BR />";
                
				$headers       = imap_fetchheader($mbox, $_msguid, $_uidtype);
				$message       = imap_body($mbox, $_msguid, $_uidtype);

				$fixedheader   = preg_replace("#(\r\n|\r|\n)#s", SWIFT_CRLF, $headers); 
				$data          = $fixedheader . preg_replace("#(\r\n|\r|\n)#s", SWIFT_CRLF, $message); 

				unset($headers); 
				unset($fixedheader); 
				unset($message);
				
				$accepted++;
				$allQueuesAccepted++;
				parseRawMimeData($data);
				
				echo "{$_msguid}: Deleting message from server...<BR />";
				
				if ($ispop3)
				{
					imap_delete($mbox, $_msguid);
				}
				else
				{
					imap_delete($mbox, $_msguid, $_uidtype);
				}				
			}
		}
		
		if (0 < $totalmsg)
		{
			echo "Processed {$totalmsg} messages for Queue ID '" . $dbCore->Record4["emailqueueid"] . "'; Accepted: {$accepted} / Rejected: {$rejected}<BR />";
		}

		imap_expunge($mbox);
		imap_close($mbox);
		
		// remove "mailbox is empty" notices from the beginning of a queue behavior when a later box is broken
		imap_errors();
		
	} else {
		
		$imap_errors = '<ol>';
		foreach(imap_errors() as $er) { $imap_errors .= "<li>$er</li>"; }
		$imap_errors .= '</ol>';
		
//		$imap_errors = print_r(imap_errors(), true);
		echo "<div style=\"border: 3px solid #600; background-color: #fee; margin: 1em 2em; padding: 0.5em 1em; text-align: justify;\">One or more errors occurred when reading Queue #{$dbCore->Record4['emailqueueid']}.<hr/>$imap_errors</div>";
	}
}

$allQueuesTotal = $allQueuesAccepted + $allQueuesRejected;
echo "All queues processed! Total messages: {$allQueuesTotal}. Total accepted: {$allQueuesAccepted} / Total rejected: {$allQueuesRejected}<BR />";

global $cron;
$cron->logentry($_SWIFT["cronid"], "success");

parserShutDown();
?>