<?php

if (!defined("INSWIFT")) {
    trigger_error("Unable to process " . htmlentities($_SERVER['PHP_SELF']), E_USER_ERROR);
}





 /*********
  *
  *  LoopControl.php
  *  ---------------
  *
  *  This block of source proffers tools to cut mail loops, to prevent servers from getting stuck in conversations
  *  with external, non-RFC compliant mail servers on auto-responder patterns and similar.  Under rare, extreme
  *  circumstances this pattern has been known to significantly degrade quality of service; this module is intended
  *  to prevent that situation.
  *
  *  @package       MailParser
  *  @subpackage    LoopControl
  *  @category      Utilities
  *
  *  @todo          KACT 5/pages/314
  *
  *  @todo          0.2     Mechanism to track email hits
  *  @todo          0.3     Mechanism to clean up too-old email hits
  *  @todo          0.4 !   Mechanism for creating tables in updated KSS install
  *  @todo          0.5     Mechanism for populating table with default configuration
  *  @todo          0.6 ~   marker() tags to verify execution
  *  @todo          0.7 ~   Trigger hooks in parser loop
  *  @todo          0.8     Mechanism for user customization of threshholds
  *  @todo          0.9     Mechanism for escape clause (click link or reply with text to un-loop-cut)
  *  @todo          0.10    Loop cut removal after X to allow mail server access again (length of cut set by which threshhold hit?)
  *  @todo          0.11    Unit tests
  *  @todo          0.12    Candidate
  *
  *  @version       0.1.3   Started Feb 1 2008
  *
  */





// todo move this to core branch after testing
// define(MARKER_MODE, "toFile");
// require_once('marker.php');





  function StartLoopBlockCheck($lcid) {  // loop cutter id, basically just the email address
	
    RemoveObsoleteEmailContactTimes();
	RemoveObsoleteLoopBlocks();

	InsertNewLoopCutterEmailContactTimeFor($lcid);
	
    if (CheckIfAddressIsBlockedByLoopCutter($lcid)) {
      return true;				
    } else {
      if (($blocklength = CheckEmailAddressForLoop($lcid)) !== false) {
        AddLoopBlock($lcid, $blocklength);
        return true;
      }
    }
    
    return false;

  }





 /*********
  *
  *  Blocks an address for a given timeframe.
  *
  *  @author    John Haugeland
  *  @since     1
  *  @version   1
  *  @access    public
  *
  *  @param     string		$address				the address to block
  *  @param		integer		$blockLengthInSeconds	the length of time until the block is removed
  *
  *  @todo      pull into a more appropriate location (after finding it)
  *
  */

  function AddLoopBlock($address, $blockLengthInSeconds) {
    
    global $dbCore;

    $RestoreBy = $blockLengthInSeconds + mktime();
    $dbCore->query("insert into " . TABLE_PREFIX . "emailloopcutsinplace(address, restoretime) values('" . $dbCore->escape($address) . "', $RestoreBy)");
    
    // comeback todo send notice to cut victim    
    // todo add error handling behavior
    
  }





  function AddLoopRule($timeframe, $maxhits, $restoretime) {
    
    global $dbCore;
	$dbCore->query("insert into " . TABLE_PREFIX . "emailloopthreshholds(length, maxhits, restoreafter) values(" . $dbCore->escape($timeframe) . ", " . $dbCore->escape($maxhits) . ", " . $dbCore->escape($restoretime) . ")");
    
  }





  function RemoveObsoleteLoopBlocks() {

    global $dbCore;
    $now = mktime();
    $dbCore->query("delete from " . TABLE_PREFIX . "emailloopcutsinplace where restoretime < $now");
    
  }





  function RemoveAllLoopBlocksForAddress($address) {

    global $dbCore;	
    $dbCore->query("delete from " . TABLE_PREFIX . "emailloopcutsinplace where address like '" . $dbCore->escape($address) . "'");

  }





  function RemoveLoopBlockById($lbid) {
	
	global $dbCore;	
	$dbCore->query("delete from " . TABLE_PREFIX . "emailloopcutsinplace where emailloopcutsinplaceid = " . $dbCore->escape($lbid));
	
  }





  function RemoveLoopRuleById($lrid) {
	
	global $dbCore;	
	$dbCore->query("delete from " . TABLE_PREFIX . "emailloopthreshholds where emailloopthreshholdsid = " . $dbCore->escape($lrid));
	
  }





 /*********
  *
  *  Creates an RFC 4122 compliant v4 UUID as defined in sec 4.1.2, returned as a string
  *
  *  This function generates an RFC compliant UUID under the UUID version 4 standard ruleset defined in section 4.1.2
  *  of RFC 4122.  This function was released to the public on May 9 2006 by David Holmes of CFD software, as found
  *  in the PHP manual notes to uniqid.  (Added $nodash)
  *
  *  @author    David Holmes
  *  @since     0.4
  *  @version   1
  *  @access    public
  *
  *  @param     boolean     $nodash      whether or not to include hyphens in the result value
  *
  *  @return    UUID as string
  *
  *  @todo      pull into a more appropriate location (after finding it)
  *
  */

  function uuid4($nodash = false) {

    $formatString = $nodash? '%04x%04x%04x%03x4%04x%04x%04x%04x' : '%04x%04x-%04x-%03x4-%04x-%04x%04x%04x';

    return sprintf($formatString,
        mt_rand(0, 65535), mt_rand(0, 65535), // 32 bits for "time_low"
        mt_rand(0, 65535), // 16 bits for "time_mid"
        mt_rand(0, 4095),  // 12 bits before the 0100 of (version) 4 for "time_hi_and_version"
        bindec(substr_replace(sprintf('%016b', mt_rand(0, 65535)), '01', 6, 2)),
        // 8 bits, the last two of which (positions 6 and 7) are 01, for "clk_seq_hi_res"
        // (hence, the 2nd hex digit after the 3rd hyphen can only be 1, 5, 9 or d)
        // 8 bits for "clk_seq_low"
        mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535) // 48 bits for "node"
    );

  }





 /*********
  *
  *  Tests uuid4() implementation
  *
  *  @author    John Haugeland
  *  @since     0.4
  *  @version   1
  *  @access    public
  *
  *  @return    Boolean (as with all test functions)
  *
  */

  function uuid4_test() {

    // todo stub until testing rig is running

    // assert length of uuid4 is 36
    // assert length of uuid4(false) is 32
    // gen two uuid4s and assert they're inequal ;; REPEATABLE

    return true;

  }





 /*********
  *
  *  Generates a short ID digest string for unique mail identification.
  *
  *  This function generates a short digest id based on the MD5 hash of RFC4122 compliant v4 UUIDs to reduce the
  *  likelihood of a collision.  Digest IDs are not immune to collision, and should still be checked for uniqueness
  *  before use.  Algorithm takes a UUID (32 hex as string), md5 hashes it (to 16 octets), splits it into eight octet
  *  pairs, renders the pairs into character ordinal tuples, XORs the tuple pairs together into four octet doublets,
  *  and renders the four doublets as a sixteen character hex representation integer string.  MD5 is just to broaden
  *  the hash gamut distribution, to speed up b-tree queries in mysql; we don't care that it's partially cracked (as
  *  it's not in security use, just utility use.)
  *
  *  @author    John Haugeland
  *  @since     0.4
  *  @version   1
  *  @access    public
  *
  *  @return    Mail hash, 16 hex element octet string
  *
  */

  function genMailId() {

    $hchunks = str_split( md5(uuid4(true), true), 2 );               // 2-byte chunks in hexadecimal
    $ochunks = array();                                              // output chunks, as in after-the-XOR

    for ($i=0, $iC=count($hchunks); $i<$iC; $i+=2) {                 // walk every other space
        $ochunks[] =
            (chr(ord($hchunks[$i][0]) ^ ord($hchunks[$i+1][0]))) .   // combine each octet tuple w/ immediate neighbor
            (chr(ord($hchunks[$i][1]) ^ ord($hchunks[$i+1][1])));    // kinda cheesy, sure, but fast
    }

    return bin2hex( implode('', $ochunks) );

  }





 /*********
  *
  *  Tests genMailId() implementation
  *
  *  @author    John Haugeland
  *  @since     0.4
  *  @version   1
  *  @access    public
  *
  *  @return    Boolean (as with all test functions)
  *
  */

  function genMailId_test() {

    // todo stub until testing rid is running

    // assert length of genMailId is 16
    // assert genMailId is hexadecimal
    // assert Id is string

  }





 /*********
  *
  *  Call wrapper to check if an incoming email address is in loop state, and to signal response halt if so.
  *
  *  Checks an incoming mail for loop conditions, particularly to cut loops with autoresponders.  This prevents one known
  *  resource shortage condition.  Indirectly SQL dependant.  This is basically a data prefetching call wrapper for
  *  TestForEmailLoopWorker, to compartmentalize dataset changes, to keep the dataset out of the algorithm itself, and
  *  to facilitate straightforward testing.
  *
  *  @author    John Haugeland
  *  @since     0.1
  *  @version   1
  *  @access    public
  *
  *  @param     string  $address     Address to be checked for loop conditions
  *
  *  @return    boolean
  *
  *  @uses      GetEmailLoopFrequencyThreshholds()
  *  @uses      GetEmailContactTimesFor()
  *  @uses      CheckForEmailLoopWorker()
  *
  *  @todo      Set marker() profile
  *
  */

  function CheckEmailAddressForLoop($address) {

    $comparetime = mktime();                                  // cache entry time for equal comparisons
    $emailtimes  = GetEmailContactTimesFor($address);
    $threshholds = GetEmailLoopFrequencyThreshholds();

    return CheckForEmailLoopWorker($comparetime, $emailtimes, $threshholds);

  }





 /*********
  *
  *  Email loop cutter algorithm implementation.  Externally testable.  SQL independant.
  *
  *  Tests an incoming dataset representing email for loop conditions, particularly to cut loops with autoresponders.  This
  *  prevents one known resource shortage condition.  This is just the actual testing algorithm; the data to be tested is
  *  fetched upstream.  SQL independant.
  *
  *  @author    John Haugeland
  *  @since     0.1
  *  @version   1
  *  @access    public
  *
  *  @param     timestamp               $comparetime     Timestamp to be used as comparison
  *  @param     ref:array(mktime(),...) $emailtimes      Key-ignorant array of mktime() timestamps
  *  @param     ref:array(thresh,...)   $threshholds     Array of arrays, internal arrays are k=>v maps of threshhold data, see GetEmailLoopFrequencyThreshholds()
  *
  *  @return    boolean
  *
  *  @todo      Speed may be improvable by sorting the contact times and frequency threshholds, then aborting inner loops to
  *             skip repeated unnessecary comparisons.  It remains unclear whether such optimization is worth the effort;
  *             this is low demand code.
  *
  *  @todo      Set marker() profile
  *
  */

  function CheckForEmailLoopWorker($comparetime, &$emailtimes, &$threshholds) {

    foreach ($emailtimes as $etime) {                         // iterate over stored hit times

      $timediff = $comparetime - $etime;                        // cache difference between comparison and current stored time
      foreach ($threshholds as &$th) {                          // iterate over threshholds (as reference so that internal member may be updated)

        if ($timediff < $th['length']) {                          // if the timestamp difference is smaller than the length marker (ie, within the threshhold)
          isset ($th['hit'])? ++$th['hit'] : $th['hit'] = 1;        // if there's a hit counter, increment it; otherwise create it at value 1
          if ($th['hit'] >= $th['maxhits']) {                       // if the hit counter is over tolerance,  
            return $th['restoreafter'];                               // return the length of the expected cut
          }
        }

      }                                                           // otherwise try next threshhold
    }                                                           // otherwise try next timestamp

    return false;                                             // otherwise exit, returning false (that no cut was discovered)

  }





  function CheckForEmailLoopWorker_test() {
    // todo
    // fill out with a series of example data and test results
  }





 /*********
  *
  *  Inserts a new mail contact time into the table of mail contact times.
  *
  *  This function takes an email address and adds the time of contact for that address to the table of kept
  *  contact times.  This function is used to keep the dataset used to determine whether a mail loop is in
  *  progress.  Requires cleanup as provided by RemoveObsoleteContactTimes().  Directly SQL dependant.
  *
  *  @author    John Haugeland
  *  @since     0.1.2
  *  @version   1
  *  @access    public
  *
  *  @param     string  $address     Address to be tracked
  *
  *  @uses      SQL
  *
  *  @todo      Set marker() profile
  *  @todo      KACT 5/pages/317
  *  @todo      Integration
  *
  */

  function InsertNewLoopCutterEmailContactTimeFor($address) {

    global $dbCore;

    // todo add error checking behavior
    $dbCore->query("insert into " . TABLE_PREFIX . "emailloopcontacttimes(address, contacttime) values('" . $dbCore->escape($address) . "', " . mktime() . ");");

  }





 /*********
  *
  *  Checks if a mail address has a loop blockage in place
  *
  *  This function checks whether or not a
  *
  *  @author    John Haugeland
  *  @since     0.1.2
  *  @version   1
  *  @access    public
  *
  *  @param     string  $address     Address to be tracked
  *
  *  @uses      SQL
  *
  *  @todo      Set marker() profile
  *  @todo      KACT 5/pages/317
  *  @todo      Integration
  *
  */

  function CheckIfAddressIsBlockedByLoopCutter($address) {
    global $dbCore;
    $blockCount = $dbCore->queryFetch("select count(*) from " . TABLE_PREFIX . "emailloopcutsinplace where address like '" . $dbCore->escape($address) . "'");
    return $blockCount['count(*)'] > 0;
  }






 /*********
  *
  *  Removes all over-threshhold times from the table of mail contact times.
  *
  *  This function removes all email contact times from the database of contact times which are older than
  *  any current threshhold limit (and therefore cannot count towards a loop cutting).
  *
  *  @author    John Haugeland
  *  @since     0.1.2
  *  @version   1
  *  @access    public
  *
  *  @param     string  $AgeInSeconds     Age of contact times considered to be too old.
  *
  *  @uses      SQL
  *
  *  @todo      Set marker() profile
  *  @todo      KACT 5/pages/317
  *  @todo      Integration
  *
  */

  function RemoveLoopControlEmailContactTimesOlderThan($AgeInSeconds) {

    global $dbCore;

    // todo add error checking behavior
    $now = mktime();
    $old = $now - $AgeInSeconds;

    $dbCore->query("delete from " . TABLE_PREFIX . "emailloopcontacttimes where contacttime < $old");

  }





 /*********
  *
  *  Digs through user preferences to find the user's maximum checked loop tolerance.  Directly SQL dependant.
  *
  *  @author    John Haugeland
  *  @since     0.3
  *  @version   1
  *  @access    public
  *
  *  @return    void
  *
  *  @uses      RemoveLoopControlEmailContactTimesOlderThan()
  *  @uses      GetMaxContactAgeTrackedByLoopCutter()
  *
  *  @todo      Set marker() profile
  *  @todo      KACT 5/pages/317
  *  @todo      Integration
  *
  */

  function GetMaxContactAgeTrackedByLoopCutter() {

    // todo add tests, add checks for failure behavior

    global $dbCore;	
    $qwork = $dbCore->queryFetch("select length from " . TABLE_PREFIX . "emailloopthreshholds order by length desc limit 1");
    return intval($qwork['length']);

  }





 /*********
  *
  *  Clears obsolete contacts from the loop cutter database; handles digging up user's configuration for obsolete
  *  timing, etc.  Indirectly SQL dependant.  This function is periodic maintenance.
  *
  *  @author    John Haugeland
  *  @since     0.3
  *  @version   1
  *  @access    public
  *
  *  @return    void
  *
  *  @uses      RemoveLoopControlEmailContactTimesOlderThan()
  *  @uses      GetMaxContactAgeTrackedByLoopCutter()
  *
  *  @todo      Set marker() profile
  *  @todo      KACT 5/pages/317
  *  @todo      Integration
  *
  */

  function RemoveObsoleteEmailContactTimes() {
	RemoveLoopControlEmailContactTimesOlderThan(GetMaxContactAgeTrackedByLoopCutter());
  }





 /*********
  *
  *  Gets the times at which this email address has recently made inbound mail contact.  Should return a flat
  *  array of mktime() stamps (accurate to one second.)  Directly SQL dependant.
  *
  *  @author    John Haugeland
  *  @since     0.1.1
  *  @version   3
  *  @access    public
  *
  *  @param     string  $address     Address to be checked for loop conditions
  *
  *  @return    array( mktime(), mktime(), ... ), empty array is valid; false on error
  *
  *  @uses      SQL
  *
  *  @todo      Set marker() profile
  *  @todo      KACT 5/pages/317
  *  @todo      Integration
  *
  */

  function GetEmailContactTimesFor($address) {

    global $dbCore;	
    $dbCore->query("select contacttime from " . TABLE_PREFIX . "emailloopcontacttimes where address like '" . $dbCore->escape($address) . "'");

    $thisrow = false;
    $output  = array();
    while ($thisrow = $dbCore->nextRecord()) {
      $output[] = intval($thisrow['contacttime']);
    }

    return $output;

    // todo add testing, error handling behavior

  }





 /*********
  *
  *  Gets the user-configurable threshholds for email loop tolerance as an array of array(length:seconds, maxhits:count, restoreafter:seconds).
  *  All returned parameters are unsigned integers.  Directly SQL dependant.
  *
  *  @author    John Haugeland
  *  @since     0.1.1
  *  @version   4
  *  @access    public
  *
  *  @return    array( array('length'=>seconds:unsigned int, 'maxhits'=>count:unsigned int, 'restoreafter'=>seconds:unsigned int), ... ), false on error
  *
  *  @uses      SQL
  *
  *  @todo      Set marker() profile
  *  @todo      KACT 5/pages/317
  *  @todo      Integration
  *
  */

  function GetEmailLoopFrequencyThreshholds() {

    global $dbCore;	
    $dbCore->query("select length, maxhits, restoreafter from " . TABLE_PREFIX . "emailloopthreshholds");

    $output = array();
    while ($thisrow = $dbCore->nextRecord()) {
      $output[] = array( length=>intval($thisrow['length']) , maxhits=>intval($thisrow['maxhits']), restoreafter=>intval($thisrow['restoreafter']) );
    }

    return $output;

    // todo add testing, error handling behavior

  }





?>