<?php
//=======================================
//###################################
// Kayako Infotech Ltd. - SWIFT Framework
//
// Source Copyright 2001-2008 Kayako Infotech Ltd.
// Unauthorized reproduction is not allowed
// License Number: $%LICENSE%$
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//                   www.kayako.com
//###################################
//=======================================

if (!defined("INSWIFT")) {
	trigger_error("Unable to process " . htmlentities($_SERVER['PHP_SELF']), E_USER_ERROR);
}

require_once("./includes/functions.php");
require_once("./includes/functions_html.php");
require_once ("./modules/parser/functions_parser_loopcontrol.php");

// Used by processBreaklines
// RML
define ("PROCESS_BREAKLINES_REGEX", "regex:");

/**
* Debug Parser Function
*/
function parserDebug($debugstr)
{
	global $_SWIFT;

	if (!$_SWIFT["clidebug"])
	{
		return false;
	}

	$_SWIFT["_debugdata"] .= $debugstr."\n";

	$fp = fopen("./files/".$_SWIFT["dbgfilename"], "a+");
	fwrite($fp, "[". edate($_SWIFT["settings"]["dt_datetimeformat"], time()) ."] ".$debugstr.SWIFT_CRLF);
	fclose($fp);

	return true;
}

/**
* Takes raw email MIME data (headers + data) and parses it into a ticket object (or reports an error if it is unable to)
*
* shouldchecksize: Boolean that tells this function whether or not it needs to verify that the message
* size does not exceed the maximum allowed.  This argument is necessary because the cron parser already
* checks message sizes, but the cli/pipe parser does not.
*/
function parseRawMimeData($mimedata, $shouldchecksize = false)
{
	global $_SWIFT, $tickets, $settings, $dbCore, $template;

	parserDebug("PARSER: Initiating MIME Classes");
	$mimeparser = new MimeParser();

	// Lets make sure the data has proper Line Feeds
	$mimedata = preg_replace("#(\r\n|\r|\n)#s", SWIFT_CRLF, $mimedata);

	parserDebug("PARSER: Decoding Mail");
	$rawstructure = $mimeparser->decodeMail($mimedata);

	parserDebug("PARSER: Mail Decoded");
	// ======= LOAD THE PARSER RULES =======
	loadParserRules();

	$_email = array();
	$_email["mimedata"] = $mimedata;
	unset($mimedata);

	// Load default values into structure
	$_email["fromName"] = trim($rawstructure->fromName);
	$_email["fromEmail"] = strtolower(trim($rawstructure->fromEmail));
	$_email["fromNameRule"] = trim($rawstructure->fromName);
	$_email["fromEmailRule"] = strtolower(trim($rawstructure->fromEmail));

	$_email["toName"] = trim($rawstructure->toName);
	$_email["toEmail"] = strtolower(trim($rawstructure->toEmail));

	$_email["replytoName"] = trim($rawstructure->replytoName);
	$_email["replytoEmail"] = strtolower(trim($rawstructure->replytoEmail));

	if ($_SWIFT["settings"]["pr_parsereplyto"] == 1 && trim($_email["replytoEmail"]) != "")
	{
		$_email["fromEmail"] = $_email["replytoEmail"];
		if (trim($_email["replytoName"]) != "")
		{
			$_email["fromName"] = $_email["replytoName"];
		}
	}

	$_email["subject"] = trim($rawstructure->subject);
	$_email["returnAddress"] = trim($rawstructure->returnAddress);
	$_email["returnAddressName"] = trim($rawstructure->returnAddressName);
	$_email["returnAddressEmail"] = trim($rawstructure->returnAddressEmail);
	$_email["recipients"] = $rawstructure->recipientAddresses;

	$_email["text"] = $rawstructure->text;
	$_email["textSize"] = $rawstructure->textSize;
	$_email["html"] = $rawstructure->html;
	$_email["htmlSize"] = $rawstructure->htmlSize;
	$_email["inReplyTo"] = $rawstructure->inReplyTo;

	// Is From Name empty?
	if (trim($_email["fromName"]) == "")
	{
		$_email["fromName"] = $_email["fromEmail"];
	}

	// Is To Name Empty?
	if (trim($_email["toName"]) == "")
	{
		$_email["toName"] = $_email["toEmail"];
	}

	// Subject empty?
	if (trim($_email["subject"]) == "")
	{
		$_email["subject"] = $_SWIFT["language"]["nosubject"];
	}
	
	if ($_SWIFT["clidebug"])
	{
		ob_start();
		print_r($rawstructure);
		$_debugcontents = ob_get_contents();
		ob_end_clean();
		parserDebug("STRUCTURE: ".$_debugcontents);
		unset($_debugcontents);
	}

	if ($_SWIFT["clidebug"])
	{
		parserDebug("MIME: ".$_email["mimedata"]);
	}

	parserDebug("PARSER: Mail Decoded. From Name: ". $_email["fromName"] .", From Email: ". $_email["fromEmail"] .", Subject: ". $_email["subject"] .", To Email: ". $_email["toEmail"] .", Text Size: ". $_email["textSize"] .", HTML Size: ".$_email["htmlSize"]);

    if (true === $shouldchecksize)
    {
        // Check the size of the message; if it exceeds the maximum size setting, ignore it.
        $msgsize = strlen($_email["mimedata"]) / 1024;

        if ($msgsize > $_SWIFT["settings"]["pr_sizelimit"])
        {
            parserDebug("PARSER: Message size ({$msgsize}KiB) exceeds the maximum allowed size of " . $_SWIFT["settings"]["pr_sizelimit"] . "KiB; shutting down!");
            doParserLog(PARSER_FAILURE, 0, 0, 0, $_email["subject"], $_email["fromEmail"], $_email["toEmail"], sprintf($_SWIFT["language"]["errtoobig"], $_SWIFT["settings"]["pr_sizelimit"] . $_SWIFT["language"]["kb"], sprintf("%.02f", $msgsize) . $_SWIFT["language"]["kb"]), 0, LOG_TICKET, $_email);
            return false;
        }
    }

	// Process Content-Type
	if (!empty($rawstructure->headers["content-type"]))
	{
		$_contenttype = processContentType($rawstructure->headers["content-type"]);
		$_email["charset"] = $_contenttype["charset"];
	}
	$_email["transferencoding"] = $rawstructure->headers["content-transfer-encoding"];
	parserDebug("PARSER: Processed Charset (". $_email["charset"] .") and Transfer Encoding (". $_email["transferencoding"] .")");

	// Process Attachments
	$_email["attachments"] = $_email["attachmentNames"] = $_email["attachmentSizes"] = array();
	$attachments = &$rawstructure->attachments;
	$_attachmentcache = $_SWIFT["attachmenttypescache"];
  $totalsize = $_email["totalAttachmentSize"] = 0;

  for ($ii=0; $ii<count($attachments); $ii++) {
	  if ($_SWIFT["settings"]["tickets_resattachments"] == 1)
	  {
		  $ext = getFileExtension($attachments[$ii]["filename"]);
		  if (is_array($_attachmentcache) && !in_array($ext, $_attachmentcache))
		  {
			  // Continue because this attachment isn't in our allowed list
			  continue;
		  }
	  }

	  $itemsize = strlen($attachments[$ii]["data"]);
	  $totalsize += $itemsize;
	  $_email["attachments"][$ii]["contents"] = $attachments[$ii]["data"];
	  $_email["attachments"][$ii]["filesize"] = $itemsize;
	  $_email["attachments"][$ii]["filename"] = $attachments[$ii]["filename"];
	  $_email["attachments"][$ii]["contenttype"] = $attachments[$ii]["contenttype"];
	  $_email["attachmentNames"][] = $attachments[$ii]["filename"];
	  $_email["attachmentSizes"][] = $itemsize;
	  $_email["totalAttachmentSize"] += $itemsize;

	  $attachments[$ii] = 0; // Free Memory
  }

  parserDebug("PARSER: Processed Attachments. Total Attachments in Email: ".count($_email["attachments"]).", Total Attachment Size: ".formattedSize($totalsize));
	
	// Unset all data heavy variables to free up some memory
	unset($attachments);
	unset($rawstructure);

	if (count($_email["recipients"]) <= 0)
	{
		parserDebug("ERROR: No Recipients Found.. Shutting Down!");
		doParserLog(PARSER_FAILURE, 0, 0, 0, $_email["subject"], $_email["fromEmail"], $_email["toEmail"], $_SWIFT["language"]["errnorecipients"], 0, LOG_TICKET, $_email);
		return false;
	}

	if (trim($_email["fromEmail"]) == "")
	{
		parserDebug("ERROR: No From Email Found.. Shutting Down!");
		doParserLog(PARSER_FAILURE, 0, 0, 0, $_email["subject"], "", $_email["toEmail"], $_SWIFT["language"]["errnofromemail"], 0, LOG_TICKET, $_email);
		return false;
	}

	// Get the Email Queue data for this email
	$_queuedata = getEmailQueueData($_email["recipients"], $_email);
	if (!$_queuedata)
	{
		parserDebug("ERROR: No Destination Email Found.. Shutting Down!");
		doParserLog(PARSER_FAILURE, 0, 0, 0, $_email["subject"], $_email["fromEmail"], $_email["toEmail"], $_SWIFT["language"]["errnoqueues"], 0, LOG_TICKET, $_email);
		return;
	}

	// Set default values from Email Queue
	$_email["queue"] = $_queuedata;
	unset($_queuedata);

	$template->setGroup($_email["queue"]["tgroupid"]);

	// Process Parser Bans
	if (isParserEmailBanned($_email["fromEmail"]))
	{
		// This email is banned!
		parserDebug("ERROR: Email is Banned.. Shutting Down!");
		doParserLog(PARSER_FAILURE, 0, 0, 0, $_email["subject"], "", $_email["toEmail"], $_SWIFT["language"]["erremailbanned"], 0, LOG_TICKET, $_email);
		return false;
	}

	// Check for ticket id or mask id in subject, if found check for existance of ticket
	$ticketiddata = array();
	$ticketiddata = parseTicketID($_email["subject"]);

	$_email["staffmsgidcheck"] = false;
	if ($_SWIFT["settings"]["t_stmsgid"] != 1)
	{
		$_email["staffmsgidcheck"] = true; // Always true if option is disabled
	}

	if (substr($_email["inReplyTo"], 0, 6) == "staff." && $_SWIFT["settings"]["t_stmsgid"] == 1)
	{
		// Need to compare message id..
		$realmsgid = substr($_email["inReplyTo"], 6);
		$_email["inReplyTo"] = $realmsgid;
		$_msgticketid = getTicketIDFromMessageID($_email["inReplyTo"]);
		$_email["staffmsgidticketid"] = $_msgticketid;
	}

	if (empty($ticketiddata["mask"]) && empty($ticketiddata["id"]))
	{
		// It seems that the ticket id isnt in subject of this mail.. do we have an inreplyto set?
		if (!empty($_email["inReplyTo"]))
		{
			// Yup! Lets see if we can lookup ticket id
			$_msgticketid = getTicketIDFromMessageID($_email["inReplyTo"]);
			if ($_msgticketid) // We found a ticket through Message-ID! It Works! It Works!
			{
				$ticketiddata["id"] = $_msgticketid;
				parserDebug("LOOKUP: Found Ticket Using Message ID: ".$_email["inReplyTo"]);
				
			}
		}

		if (empty($ticketiddata["id"]) && $_SWIFT['settings']['t_searchticketidinbody'] == 1)
		{
		   // We try to look it up in the text body
		   $ticketiddata = parseTicketID($_email["text"]); // try to look it up in the mail body
		   parserDebug("LOOKUP: Searching Ticket ID In text Body!");
		}
		
		if (empty($ticketiddata["id"]) && $_SWIFT['settings']['t_searchticketidinbody'] == 1)
		{
		   // We try to look it up in the html body
		   $ticketiddata = parseTicketID($_email["html"]); // try to look it up in the mail body
		   parserDebug("LOOKUP: Searching Ticket ID In HTML Body!");
		}

		// Its Still Empty?!?!?! How in this world can this be possible.. we only proceed if the subject has :, otherwise its most probably a new ticket..
		if (empty($ticketiddata["id"]) && empty($ticketiddata["mask"]) && stristr($_email["subject"], ":"))
		{
			$parsedSubject = trim(substr(stristr($_email["subject"], ":"),1));
			// See if we can look it up using similar subject..
			$_usingsubject = $dbCore->queryFetch("SELECT `ticketid` FROM `". TABLE_PREFIX ."tickets` WHERE `email` = '". $dbCore->escape($_email["fromEmail"]) ."' AND `subject` = '". $dbCore->escape(substr($parsedSubject,0,200)) ."' ORDER BY `ticketid` DESC LIMIT 1;");
			if (!empty($_usingsubject["ticketid"]))
			{
				$ticketiddata["id"] = $_usingsubject["ticketid"];
				parserDebug("LOOKUP: Found Ticket ID Using Mail Subject: ".$_email["subject"]);
			}
		}

		// Ok.. I give up.. this has to be a new ticket..
	}
	
	if (!empty($ticketiddata["mask"]))
	{
		$_ticketmask = $ticketiddata["mask"];
		$_ticketid = getTicketIDFromMask($ticketiddata["mask"]);
	} else if (!empty($ticketiddata["id"])) {
		$_ticketid = $ticketiddata["id"];
	} else {
		$_ticketid = false;
	}
	
	if ($ticketiddata["isthirdparty"] == true)
	{
		$_email["isthirdparty"] = true;
	}
	
	if ($ticketiddata["isalert"] == true)
	{
		$_email["isalert"] = true;
	}

	// Cleanup the subject
	$_email["subject"] = cleanupSubject($_email["subject"]);

	// Check for merged status, this step will only be executed if there was a ticket id or mask in subject and we were unable to find it in the database
	if (!$_ticketid && (trim($ticketiddata["mask"]) != "" || trim($ticketiddata["id"]) != ""))
	{
		if (trim($ticketiddata["mask"]) != "")
		{
			// Is this ticket merged?
			if ($mergedticketid = getMergedStatusFromMask($ticketidata["mask"]))
			{
				$_ticketid = $mergedticketid;
			}
		} else if (trim($ticketiddata["id"]) != "") {
			// Is this ticket merged? (yeah again another check)
			if ($mergedticketid = getMergedStatusFromID($ticketiddata["id"]))
			{
				$_ticketid = $mergedticketid;
			}
		}
	}

	// Now we should know whether this email belongs to a ticket or not. If there is a ticketid set, try to load the given ticket.
	$_ticket = false;
	$sendautoresponder = true;

	// Before we proceed, we need to see if the email is from the queue email itself
	if (isset($_SWIFT["queuecache"]["p"][$_email["fromEmail"]]))
	{
		// We never send the autoresponder if the email belongs to the queue.. never, ever.. mail loops are evil
		$sendautoresponder = false;
	}

	if ($_ticketid)
	{
		$_ticket =& new TicketMain($_ticketid);
		$_SWIFT["ticketobj"] =& $_ticket;
	}
	
	// The final reply/ticket check, by now we would have tried every freaking thing out there to see if this email belongs to an existing ticket
	if (!empty($_ticket->ticket["ticketid"]))
	{
		// We found a ticket to which this email belongs, so we fill up the data
		$_email["isticketreply"] = true;
		$_email["ticketid"] = $_ticketid;
		$_email["dispatchticketid"] = iif($_SWIFT["settings"]["t_eticketid"]=="seq",$_ticket->ticket["ticketid"], $_ticket->ticket["ticketmaskid"]);
		$_email["dispatchsubject"] = $_ticket->getEmailSubject(SUBJECT_USER);
	} else {
		// Failed to load the ticket..
		$_email["isticketreply"] = false;
	}

	parserDebug("PARSER: ".iif($_email["isticketreply"]==true, "Is Ticket Reply for ".$_email["ticketid"], "Is a New Ticket"));

	// This is where we parse out the original email content and decide which one to store, text or html
	if ($_SWIFT["settings"]["pr_contentpriority"] == "text")
	{
		// Text is empty but we do have HTML content in there. This will happen in very rare cases
		if (trim($_email["text"]) == "")
		{
			$_email["ishtml"] = true;
			$emailcontent = $_email["html"];
		} else {
			$_email["ishtml"] = false;
			$emailcontent = $_email["text"];
		}
	} else if ($_SWIFT["settings"]["pr_contentpriority"] == "html") {
		if (trim($_email["html"]) == "")
		{
			$_email["ishtml"] = false;
			$emailcontent = $_email["text"];
		} else {
			$_email["ishtml"] = true;
			$emailcontent = $_email["html"];
		}
	}

	// Strip script tags if the setting is enabled.
	if ($_SWIFT["settings"]["pr_stripscript"] == 1)
	{
	    $emailcontent = stripScriptTags($emailcontent);
	}

	// Process the final content by splitting the original content using break lines.
	if ($_email["isticketreply"] == true)
	{
		$_email["finalContent"] = processBreakLines($emailcontent);
	} else {
		$_email["finalContent"] = $emailcontent;
	}
	parserDebug("PARSER: Final Content Is: ".$_email["finalContent"]);

	// Check for Flood Protection
	$_email['loopblock'] = false;
	if (StartLoopBlockCheck($_email["fromEmail"]) == true) {

		parserDebug('WARNING: Loop Cutter Flood Protection Triggered');
		$_email['loopblock'] = true;
		
		if ($_SWIFT['settings']['pr_loopcut_ignores_cut_mail']) {
			
			parserDebug('ERROR: Flood protection / loop cutter rules have triggered; email is rejected.  No auto-response was sent, and no ticket was created.');
			doParserLog(PARSER_FAILURE, $_ticketid, $_ticketmask, $_email["queue"]["emailqueueid"], $_email["subject"], $_email["fromEmail"], $_email["toEmail"], $_SWIFT["language"]["errloopcontrol"], 0, LOG_TICKET, $_email);
			return false;
						
		} else if ($_SWIFT['settings']['pr_loopcut_prevents_autoresponder']) {
			
			parserDebug('WARNING: Loop cutter prevented the dispatch of an automatic response.  Ticket was created as normal.');
			$sendautoresponder = false;
			
		} else {
			parserDebug('WARNING: Flood protection / loop cutter took no action!  Configuration of loop control settings allows accepted message and does not prevent auto-response.');				
		}

	}

	// Check for staff reply
	$_email["isstaffreply"] = false;
	if ($_SWIFT["settings"]["t_pstaffreply"] == 1 && $_email["isticketreply"] == true && $_email["isthirdparty"] != true && $_email["isalert"] == true)
	{
		if (isStaffEmail($_email["fromEmail"]))
		{
			parserDebug("PARSER: Reply is by Staff");
			if ((!$_email["staffmsgidcheck"] && $_email["staffmsgidticketid"] == $_ticket->ticket["ticketid"] && !empty($_email["staffmsgidticketid"])) || $_email["staffmsgidcheck"] == true)
			{
				parserDebug("PARSER: Staff MSGID Check is Success");
				$_email["isstaffreply"] = true;
			} else {
				parserDebug("PARSER: Staff MSGID Check is Failure");
			}
		}
	}

	// ======= RUN THE PARSER RULES =======
	$parserruleidlist = runParserRules($_email, $_ticket);

	executeParserRuleActions($parserruleidlist, RULE_PREPARSE, $_email, $_ticket);
	if ($_email["ignore"] == true)
	{
		return false;
	}

	if ($_email["noautoresponder"] == true)
	{
		parserDebug("PARSER: Not sending auto responder as it is disabled in queue");
		$sendautoresponder = false;
	} else {
		parserDebug("PARSER: Auto responder is enabled in queue");
	}
	
	// Set the template group
	$template->setGroup($_email["queue"]["tgroupid"]);
	$template->loadLanguageSection("default", TEMPLATE_DB);
	$template->loadLanguageSection("ticketemails", TEMPLATE_DB);

	// Create a New Customer Reply
	if ($_email["isticketreply"] == true && $_email["isstaffreply"] == false)
	{
		$userid = getUserEmail($_email["fromEmail"]);
		if (!$userid) // Do we need to add the user to recipients?
		{
			$add = true;
			$isrecipient = false;
			$_recipients = $_ticket->getRecipients();
			if (is_array($_recipients))
			{
				foreach ($_recipients as $key=>$val)
				{
					if ($val["email"] == $_email["fromEmail"])
					{
						$isrecipient = true;
						$add = false;
					}
				}
			}

			if ($add == true)
			{
				$isrecipient = true;
				$_ticket->addRecipients(array($_email["fromEmail"]), RECIPIENT_CCUSER);
			}
		}
		$_ticket->resetSLATimeline();

		if ($_email["isthirdparty"] == true)
		{
			$posttype = POST_THIRDPARTY;
		} else if ($isrecipient == true) {
			$posttype = POST_RECIPIENT;
		} else {
			$posttype = POST_CLIENT;
		}

		// Create the Ticket Post
		$ticketpostresult = $_ticket->createTicketPost(false, $posttype, $userid, "0", $_email["fromName"], $_email["fromEmail"], "", $_email["subject"], $_email["finalContent"], $_email["ishtml"], true, $_email["attachments"]);
		$ticketpostid = $ticketpostresult[0];
		$messageid    = $ticketpostresult[1];
		
		$_ticket->messageid = $messageid;

		if (empty($_SWIFT["settings"]["t_cstatusupd"]))
		{
//			$_ticket->resetTemplateGroupStatus();
		}

		parserDebug("SUCCESS: Created Post: ".$ticketpostid);

		doParserLog(PARSER_SUCCESS, $_ticketid, $_ticketmask, $_email["queue"]["emailqueueid"], $_email["subject"], $_email["fromEmail"], $_email["toEmail"], sprintf(iif($_email["isthirdparty"]==true,$_SWIFT["language"]["sctcreatedreply"], $_SWIFT["language"]["scccreatedreply"]), $ticketpostid, $_email["dispatchticketid"]), 0, LOG_REPLY, $_email, $ticketpostid);

	// Create a New Staff Reply
	} else if ($_email["isticketreply"] == true && $_email["isstaffreply"] == true) {

		foreach ($_SWIFT["staffcache"] as $key=>$val)
		{
			if (strtolower($val["email"]) == strtolower($_email["fromEmail"]))
			{
				$_staffid = $val["staffid"];
				break;
			}
		}

		if ($_staffid)
		{
			// Create the Ticket Post for Staff
			$ticketpostid = $_ticket->createStaffReply($_staffid, $_email["fromName"], $_email["fromEmail"], $_email["subject"], $_email["finalContent"], true, $_email["queue"]["emailqueueid"], $_email["attachments"], "", false, "", false);

			// Reset status to default template group one..
			$_ticket->resetTemplateGroupStatus();

			parserDebug("SUCCESS: Created Staff Post: ".$ticketpostid);

			doParserLog(PARSER_SUCCESS, $_ticketid, $_ticketmask, $_email["queue"]["emailqueueid"], $_email["subject"], $_email["fromEmail"], $_email["toEmail"], sprintf($_SWIFT["language"]["scccreatedstaffreply"], $ticketpostid, $_email["dispatchticketid"]), 0, LOG_REPLY, $_email, $ticketpostid);
		}
	// Create a New Ticket
	} else {
		// Check to see if registration required is enabled for this queue
		$userid = getUserEmail($_email["fromEmail"]);
		if ($_email["queue"]["registrationrequired"] == 1 && !$userid)
		{
			// ======= BEGIN MAIL SEND CODE =======
			// Hard Coded Bounce Checks, No other way to work around this...
			if (!stristr($_email["fromEmail"], "MAILER-DAEMON") && !stristr($_email["subject"], "failure notice") && !stristr($_email["subject"], "Returned Mail"))
			{
				require_once ("./includes/functions_mail.php");

				$template->setGroup(iif(empty($_email["queue"]["tgroupid"]), 1, $_email["queue"]["tgroupid"]));
				$template->assign("ishtml", true);
				$_templateresulthtml = $template->displayTemplate("email_noregistration", TEMPLATE_DB);
				$template->assign("ishtml", false);
				$_templateresulttext = $template->displayTemplate("email_noregistration", TEMPLATE_DB);

				$mailObj = new SWIFT_Mail();

				$mailObj->setSubject($_SWIFT["language"]["regemailrejectsub"]);
				$mailObj->setData($_templateresulthtml, $_templateresulttext);

				$mailObj->sendMail($_email["fromEmail"], $_SWIFT["settings"]["general_returnemail"], $_SWIFT["settings"]["general_companyname"], false);
			}
			// ======= END MAIL SEND CODE =======

			// Check for the email in registered users table
			doParserLog(PARSER_FAILURE, $_ticketid, $_ticketmask, $_email["queue"]["emailqueueid"], $_email["subject"], $_email["fromEmail"], $_email["toEmail"], sprintf($_SWIFT["language"]["errusernotreg"], htmlspecialchars($_email["fromEmail"])), 0, LOG_TICKET, $_email);
			return false;
		}
       // Check to see if registration required is enabled for this queue, and the from e-mail address belongs to a validated user account
        else if ( $userid  && ($_email["queue"]["registrationrequired"] == 1) && (getUserValidationStatus($_email["fromEmail"]) == 0) )
        {
            if (!stristr($_email["fromEmail"], "MAILER-DAEMON") && !stristr($_email["subject"], "failure notice") && !stristr($_email["subject"], "Returned Mail"))
            {
                require_once ("./includes/functions_mail.php");

                $template->setGroup(iif(empty($_email["queue"]["tgroupid"]), 1, $_email["queue"]["tgroupid"]));

                if ( $_SWIFT["settings"]["u_enablesveri"] == 1 )
                {
                    $template->assign("ishtml", true);
                    $_templateresulthtml = $template->displayTemplate("email_needsstaffvalidation", TEMPLATE_DB);
                    $template->assign("ishtml", false);
                    $_templateresulttext = $template->displayTemplate("email_needsstaffvalidation", TEMPLATE_DB);
                }
                else
                {
                    $template->assign("ishtml", true);
                    $_templateresulthtml = $template->displayTemplate("email_notvalidatedgeneric", TEMPLATE_DB);
                    $template->assign("ishtml", false);
                    $_templateresulttext = $template->displayTemplate("email_notvalidatedgeneric", TEMPLATE_DB);
                }


                $mailObj = new SWIFT_Mail();

                $mailObj->setSubject($_SWIFT["language"]["validationemailrejectsub"]);
                $mailObj->setData($_templateresulthtml, $_templateresulttext);

                $mailObj->sendMail($_email["fromEmail"], $_SWIFT["settings"]["general_returnemail"], $_SWIFT["settings"]["general_companyname"], false);
            }

            doParserLog(PARSER_FAILURE, $_ticketid, $_ticketmask, $_email["queue"]["emailqueueid"], $_email["subject"], $_email["fromEmail"], $_email["toEmail"], sprintf($_SWIFT["language"]["errusernotvalidated"], htmlspecialchars($_email["fromEmail"])), 0, LOG_TICKET, $_email);
            return false;
        }

		// Register the user.
		if (!$userid)
		{
			$emailtgroup = $_email["queue"]["tgroupid"];
			$usergroupid = $_SWIFT["tgroupcache"][$emailtgroup]["regusergroupid"];
			if (empty($usergroupid))
			{
				$usergroupid = 2;
			}
			$_loginapi = LOGINAPI_DEFAULT;
			if (!empty($_SWIFT["tgroupcache"][$emailtgroup]["loginapi_moduleid"]))
			{
				$_loginapi = $_SWIFT["tgroupcache"][$emailtgroup]["loginapi_moduleid"];
			}
			$userid = insertUser(iif($_SWIFT["settings"]["u_enablesveri"]==1, false, true), $_email["fromEmail"], substr(buildHash(),0, 8), $usergroupid, $_loginapi, 0, $_email["fromName"], 0, 0, false, 1, true);
			parserDebug("SUCCESS: Created User: ".$userid);
		}
		
		// Only send the autoresponder message if the group this user belongs to has the
		// setting enabled.
		$user_db_data = $dbCore->queryFetch("SELECT usergroupid FROM `" . TABLE_PREFIX . "users` WHERE (userid='" . $dbCore->escape($userid) . "');");
		
		if ($sendautoresponder && $_SWIFT['usergroupsettingcache'][$user_db_data['usergroupid']]['perm_sendautoresp'] != 0)
		{
			$sendautoresponder = true;
		} else {
			$sendautoresponder = false;
		}

		$_ticketiddata = createTicket(POST_CLIENT, 0, $userid, $_email["fromName"], $_email["fromEmail"], mb_substr($_email["subject"], 0, 150, $_SWIFT["language"]["charset"]), $_email["finalContent"], iif($_email["ishtml"]==true, "1", "0"), $_email["queue"]["priorityid"], $_email["queue"]["ticketstatusid"], $_email["queue"]["departmentid"], $_email["queue"]["emailqueueid"], $_email["charset"], $_email["transferencoding"], $_email["queue"]["tgroupid"], $_email["attachments"], true, $sendautoresponder);

		parserDebug("SUCCESS: Created Ticket: ".$_ticketiddata[1]);

		doParserLog(PARSER_SUCCESS, $_ticketiddata[0], $_ticketiddata[1], $_email["queue"]["emailqueueid"], mb_substr($_email["subject"], 0, 150, $_SWIFT["language"]["charset"]), $_email["fromEmail"], $_email["toEmail"], sprintf($_SWIFT["language"]["scccreatedticket"], iif($_SWIFT["settings"]["t_eticketid"]=="seq",$_ticketiddata[0], $_ticketiddata[1])) . getParserLogMsg($sendautoresponder, $_email['loopblock']), 0, LOG_TICKET, $_email, $_ticketiddata[2]);
	}

	if (is_object($_SWIFT["ticketobj"]))
	{
		parserDebug("SUCCESS: Ticket Object is Valid");
		
	    // Check to see if alert rules should not be processed for this ticket
        $_SWIFT["ticketobj"]->noalertrules = ($_email["noalertrules"] == true) ? true : false;
        
		if (count($_email["recipients"]) > 1 && $_SWIFT["settings"]["t_autorecip"] == 1 )
		{
			foreach($_email["recipients"] as $recipient)
			{			
				if($recipient != $_email['fromEmail'] && $recipient != $_email['toEmail'] && $recipient != $_email["replytoEmail"] && !in_array($recipient,$_SWIFT["ticketobj"]->getRecipients()) && !isset($_SWIFT["queuecache"]["p"][$recipient]))
				{
					$_SWIFT["ticketobj"]->addRecipients(array($recipient),RECIPIENT_CCUSER);
					if ($_SWIFT['settings']['t_autorespondercc'] == 1 && $sendautoresponder)
					{
						$_SWIFT["ticketobj"]->sendAutoResponder($recipient);
					}
				}
			}
		}
	} else {
		parserDebug("FAILURE: Ticket Object is Invalid");
	}

	executeParserRuleActions($parserruleidlist, RULE_POSTPARSE, $_email, $_SWIFT["ticketobj"]);

	return true;
}

/**
* Create a parser log entry
*/
function doParserLog($status, $typeid, $ticketmaskid, $emailqueueid, $subject, $fromemail, $toemail, $description, $parsetimetaken, $responsetype, $_email, $ticketpostid=0)
{
	global $dbCore, $_SWIFT;

	if ($_SWIFT["settings"]["pr_enablelog"] != 1)
	{
		// return right here because logging is disabled
		return false;
	}

	if (empty($parsetimetaken))
	{
		$_SWIFT["endparser"] = fetchmicrotime();
		$parsetimetaken = $_SWIFT["endparser"]-$_SWIFT["startparser"];
	}

	$size = strlen($_email["mimedata"]);

	$dbCore->query("INSERT INTO `". TABLE_PREFIX ."parserlogs` (`typeid`, `ticketmaskid`, `dateline`, `emailqueueid`, `logtype`, `subject`, `fromemail`, `toemail`, `size`, `description`, `parsetimetaken`, `responsetype`, `ticketpostid`) VALUES ('". intval($typeid) ."', '". $dbCore->escape($ticketmaskid) ."', '". DATENOW ."', '". intval($emailqueueid) ."', '". $dbCore->escape(iif($status==PARSER_SUCCESS, "success", "failure")) ."', '". $dbCore->escape($subject) ."', '". $dbCore->escape($fromemail) ."', '". $dbCore->escape($toemail) ."', '". $dbCore->escape($size) ."', '". $dbCore->escape($description) ."', '". $dbCore->escape($parsetimetaken) ."', '". $dbCore->escape($responsetype)."', '". intval($ticketpostid) ."');");
	$parserlogid = $dbCore->lastInsertId();

	if (!$parserlogid)
	{
		return false;
	}

	// Check for max log size before proceeding..
	$size = $size/1024;
	$maxlogsizekb = intval($_SWIFT["settings"]["pr_maxlogsize"]);
	$maxlogsize = $maxlogsizekb*1024;

	parserDebug("VARIABLE: size: ".$size);
	parserDebug("VARIABLE: maxlogsizekb: ".$maxlogsizekb);

	if (trim($_email["mimedata"]) != "" && $size <= $maxlogsizekb)
	{
		// Do Nothing
	} else {
		$_email["mimedata"] = $_SWIFT["language"]["clearedduelimit"];
	}
	if (isset($_email["attachments"]))
	{
		unset($_email["attachments"]);
	}

	// Add the parser contents
	$dbCore->shutdownQuery("INSERT INTO `". TABLE_PREFIX ."parserlogdata` (`parserlogid`, `contents`) VALUES ('". intval($parserlogid) ."', '". $dbCore->escape(serialize($_email)) ."');");
}

/**
* Gets the Email Queue data from an array of email addresses
*/
function getEmailQueueData($recipients, &$_email)
{
	global $dbCore, $_SWIFT;

	if (count($_SWIFT["parsercatchallcache"]))
	{
		parserDebug("QUEUECHECK: Catch all rules exist");
		$catchall = true;
	} else {
		$catchall = false;
	}

	foreach ($recipients as $key=>$val)
	{
		$val = strtolower(trim($val));

		if (trim($val) != "" && isset($_SWIFT["queuecache"]["p"][$val]))
		{
			parserDebug("QUEUECHECK: Found Email Queue for: ".$val);
			$emailqueueid = intval($_SWIFT["queuecache"]["p"][$val]);

			if ($_SWIFT["queuecache"]["list"][$emailqueueid]["ticketautoresponder"] != "1")
			{
				parserDebug("QUEUECHECK: No autoresponder to dispatch");
				$_email["noautoresponder"] = true;
			}
			return $_SWIFT["queuecache"]["list"][$emailqueueid];
		} else if ($catchall == true) {
			parserDebug("QUEUECHECK: Checking Catch All rules");
			foreach ($_SWIFT["parsercatchallcache"] as $ckey=>$cval)
			{
				parserDebug("QUEUECHECK: Matching $val against ".$cval['ruleexpr']);
				if (preg_match($cval['ruleexpr'], $val)) {
					parserDebug("QUEUECHECK: Match found");
					$emailqueueid = $cval["emailqueueid"];
					if (!isset($_SWIFT["queuecache"]["list"][$emailqueueid]))
					{
						parserDebug("QUEUECHECK: No Queue Found");
						return false;
					}

					if ($_SWIFT["queuecache"]["list"][$emailqueueid]["ticketautoresponder"] != "1")
					{
						$_email["noautoresponder"] = true;
					}

					parserDebug("QUEUECHECK: Returning Email Queue ID:".$_SWIFT["queuecache"]["list"][$emailqueueid]);
					return $_SWIFT["queuecache"]["list"][$emailqueueid];
				} else {
					parserDebug("QUEUECHECK: Regular Expression didnt match");
				}
			}
		}
		
	}

	return false;
}

/**
* Processes the Content-Type and returns Mime Type and Charset.
*/
function processContentType($datastr)
{
	$returnValue=array();
	$ctypeparts = explode(';',$datastr);
	$returnValue['mimetype'] = trim($ctypeparts[0]);

	reset($ctypeparts);
	next($ctypeparts);
	while(list(,$v) = each($ctypeparts))
	{
		unset($reg);
		eregi('([^=]*)="(.*)"',$v,$reg);
		if (trim($reg[1]) && trim($reg[2])) {
			$returnValue[trim(strtolower($reg[1]))] = trim($reg[2]);
		}
	}

	return $returnValue;
}

/**
* Processes Break Lines for the given content, What it does is splits up the content into two if a break line is found and returns the splitted content. This function is useful to make sure that quoted messages do not get into the system.
*/
function processBreakLines($content)
{
	global $_SWIFT;
	
	parserDebug("NOTICE: Processing breaklines");
	$breakline = &$_SWIFT["breaklinecache"];
	
	if (!is_array($breakline))
	{
		parserDebug("NOTICE: No breaklines found!");
		return $content;
	}
		
	parserDebug("NOTICE: Breaklines found: " . count($breakline));

    $regexSpecifierLen = strlen(PROCESS_BREAKLINES_REGEX);
    
	foreach ($breakline as $val)
	{
		parserDebug("NOTICE: Processing breakline: " . $val);

		if (PROCESS_BREAKLINES_REGEX == substr($val, 0, $regexSpecifierLen))
		{
		    // This is a regular expression value.
		    parserDebug("NOTICE: Processing breakline '" . $val . "' as a regular expression.");
		    
		    $regex = substr($val, $regexSpecifierLen, strlen($val) - $regexSpecifierLen);
		    $split = preg_split($regex, $content);

            // If more than one item is in the array then the match was successful
            // and the reply is the first one.
		    if (count($split) > 1)
		    {
		        // The regular expression was successful; return the first match, as it will be all
		        // the text before the regular expression matched.
		        return $split[0];
		    }
		    
		    // If the regex doesn't match, try the next breakline.
		}
		else
		{
		    // Process normally; not a regular expression value.
		    $breakcontent = reversestrchr($content, $val);
		    
		    // The breakline matched; return the text from the beginning of the string up to the breakline.
		    if ($breakcontent)
		    {
			    return $breakcontent;
		    }
		    
		    // If it doesn't match, try the next breakline.
	    }
	}

    // No breaklines matched, so return the content as-is.
	return $content;
}

/**
* Reverse of strchr, returns the data from start of haystack to needle, Does not include the needle
*/
function reversestrchr($haystack, $needle)
{
	if (!$posi = strpos(strtoupper($haystack), strtoupper($needle)))
	{
		return false;
	} else {
		return substr($haystack,0,$posi);
	}
}

/**
* Gets a Ticket Mask ID or a Ticket ID From a subject. Allows a lot of combinations
*/
function parseTicketID($subject)
{
	global $_SWIFT;

	$resticketid = array();

	if (ereg("\[([A-Za-z0-9_ ]{1,100}) (\#|~|!)([A-Za-z]{3}-[0-9]{5,6})\]", $subject, $result) && $_SWIFT["settings"]["t_eticketid"] == "random") {
		// First try to parse a ticket mask with a 1-100 digit company code prefixed. Example: [KAYAKO #ABC-12345]: Subject
		// 1 = Company Prefix
		// 2 = Ticket Mask ID

		$resticketid["isthirdparty"] = iif($result[2]=="~",true, false);
		$resticketid["isalert"] = iif($result[2]=="!",true, false);
		$resticketid["mask"] = $result[3];
	} else if (ereg("\[(\#|~|!)([A-Za-z]{3}-[0-9]{5,6})\]", $subject, $result) && $_SWIFT["settings"]["t_eticketid"] == "random") {
		// No luck? Now try to get a simple ticket mask then. Example: [#ABC-12345]: Subject
		// 1 = Ticket Mask ID

		$resticketid["isthirdparty"] = iif($result[1]=="~",true, false);
		$resticketid["isalert"] = iif($result[1]=="!",true, false);
		$resticketid["mask"] = $result[2];
	} else if (ereg("\[([A-Za-z]{3}-[0-9]{5,6})\]", $subject, $result) && $_SWIFT["settings"]["t_eticketid"] == "random") {
		// Give another try at ticket mask, This is added for backwards compatibility. Example: [ABC-12345]: Subject
		// 1 = Ticket Mask ID

		$resticketid["mask"] = $result[1];
	} else if (ereg("\[([A-Za-z0-9_ ]{1,100}) (\#|~|!)([0-9]{5,6})\]", $subject, $result) && $_SWIFT["settings"]["t_eticketid"] == "seq") {
		// Try to get a ticket id with prefix?. Example: [KAYAKO #12345]: Subject
		// 1 = Company Prefix
		// 2 = Ticket Mask ID

		$resticketid["isthirdparty"] = iif($result[2]=="~",true, false);
		$resticketid["isalert"] = iif($result[2]=="!",true, false);
		$resticketid["id"] = $result[3];
	} else if (ereg("\[(\#|~|!)([0-9]{1,22})\]", $subject, $result) && $_SWIFT["settings"]["t_eticketid"] == "seq") {
		// Still nothing? Try to get the numeric ticket id then. Example: [#3412]: Subject
		// 1 key = 12345

		$resticketid["isthirdparty"] = iif($result[1]=="~",true, false);
		$resticketid["isalert"] = iif($result[1]=="!",true, false);
		$resticketid["id"] = $result[2];
	} else if (ereg("\[([A-Za-z0-9_ ]{1,100}) (\#|~|!)([0-9]{1,22})\]", $subject, $result) && $_SWIFT["settings"]["t_eticketid"] == "seq") {
		// Still nothing? Try to get the numeric ticket id then. Example: [PREFIX #3412]: Subject
		// 1 key = 12345

		$resticketid["isthirdparty"] = iif($result[2]=="~",true, false);
		$resticketid["isalert"] = iif($result[2]=="!",true, false);
		$resticketid["id"] = $result[3];
	}

	if ($resticketid["mask"] != "" || $resticketid["id"] != "")
	{
		return $resticketid;
	} else {
		return false;
	}
}

/**
* This function cleans up the subject of its ticket id junk and returns a plain one, very useful in case a person replies to a ticket that doesnt exist.. doesnt cause the "ticket id loop" in subjects as it used to in previous versions
*/
function cleanupSubject($subject)
{
	// First see if a subject has a ticket id in it?
	if (parseTicketID($subject) && ereg("\[(.*)\]:(.*)", $subject, $result))
	{
		// Success!
		return $result[2];
	} else {
		return $subject;
	}
}

/**
* Checks whether or not the from email is banned
*/
function isParserEmailBanned($email)
{
	global $dbCore, $_SWIFT;

	if (!_is_array($_SWIFT["parserbanscache"]))
	{
		return false;
	}

	if (in_array($email, $_SWIFT["parserbanscache"]))
	{
		return true;
	}

	return false;
}

/**
* Parses out the return value of imap_fetchstructure()
* and returns the size, in bytes of all MIME parts together (the total size of the email message).
*/
function getMessageSize($parts)
{
	$runningTotal = 0;
	
	foreach ($parts as $curPart)
	{
		$runningTotal += $curPart->bytes;
	}
	
	return $runningTotal;
}

function getParserLogMsg($sendautoresponder, $isBlockage)
{
	global $_SWIFT;
	
	// Displays the message in the parser log (Admin CP >> Mail Parser >> Parser Log)
	// If the autoresponder isn't being sent due to a loop blockage, set the message to convey that.
	// If it isn't being sent due to a setting, then let the message convey that instead.
	$logMsg = "";
	
	if (!$sendautoresponder)
	{
		if ($isBlockage)
		{
			// Loop blockage
			$logMsg = $_SWIFT['language']['pr_loopcutter_prevents_autoresponder_desc'];
		}
		else
		{
			// Not due to a loop blockage, but a setting
			$logMsg = $_SWIFT['language']['autorespondernotsent'];
		}
	}
	else
	{
		$logMsg = "";
	}
	
	return $logMsg;
}
?>